@@ -1,14 +1,15 @@
 #!/usr/bin/env python
 u"""
 compute_tide_corrections.py
-Written by Tyler Sutterley (08/2020)
+Written by Tyler Sutterley (11/2020)
 Calculates tidal elevations for correcting elevation or imagery data
 
 Uses OTIS format tidal solutions provided by Ohio State University and ESR
     http://volkov.oce.orst.edu/tides/region.html
     https://www.esr.org/research/polar-tide-models/list-of-polar-tide-models/
     ftp://ftp.esr.org/pub/datasets/tmd/
-or Global Tide Model (GOT) solutions provided by Richard Ray at GSFC
+Global Tide Model (GOT) solutions provided by Richard Ray at GSFC
+or Finite Element Solution (FES) models provided by AVISO
 
 INPUTS:
     x: x-coordinates in projection EPSG
@@ -22,7 +23,7 @@ OPTIONS:
         default: J2000 (seconds since 2000-01-01T00:00:00)
     TYPE: input data type
         drift: drift buoys or satellite/airborne altimetry (time per data point)
-        grid: spatial grids or images (single time per image)
+        grid: spatial grids or images (single time for all data points)
     TIME: time type if need to compute leap seconds to convert to UTC
         GPS: leap seconds needed
         TAI: leap seconds needed (TAI = GPS + 19 seconds)
@@ -32,7 +33,7 @@ OPTIONS:
     METHOD: interpolation method
         bilinear: quick bilinear interpolation
         spline: scipy bivariate spline interpolation
-        linear, cubic, nearest: scipy griddata interpolations
+        linear, nearest: scipy regular grid interpolations
     FILL_VALUE: output invalid value (default NaN)
 
 PYTHON DEPENDENCIES:
@@ -63,9 +64,11 @@ PROGRAM DEPENDENCIES:
     read_FES_model.py: extract tidal harmonic constants from FES tide models
 
 UPDATE HISTORY:
+    Updated 11/2020: added model constituents from TPXO9-atlas-v3
     Updated 08/2020: using builtin time operations.
         calculate difference in leap seconds from start of epoch
-    Updated 07/2020: added function docstrings, FES2014 and TPX09-atlas-v2
+        using conversion protocols following pyproj-2 updates
+    Updated 07/2020: added function docstrings, FES2014 and TPXO9-atlas-v2
         use merged delta time files combining biannual, monthly and daily files
     Updated 03/2020: added TYPE, TIME, FILL_VALUE and METHOD options
     Written 03/2020
@@ -86,13 +89,6 @@ from pyTMD.read_netcdf_model import extract_netcdf_constants
 from pyTMD.read_GOT_model import extract_GOT_constants
 from pyTMD.read_FES_model import extract_FES_constants
 
-#-- PURPOSE: convert value to numpy arrays if single point
-def point_to_array(val):
-    """
-    Convert a value to a numpy array if originally a single point
-    """
-    return np.array([val]) if (np.ndim(val) == 0) else np.copy(val)
-
 #-- PURPOSE: compute tides at points and times using tide model algorithms
 def compute_tide_corrections(x, y, delta_time, DIRECTORY=None, MODEL=None,
     EPSG=3031, EPOCH=(2000,1,1,0,0,0), TYPE='drift', TIME='UTC',
@@ -124,7 +120,7 @@ def compute_tide_corrections(x, y, delta_time, DIRECTORY=None, MODEL=None,
     METHOD: interpolation method
         bilinear: quick bilinear interpolation
         spline: scipy bivariate spline interpolation
-        linear, cubic, nearest: scipy griddata interpolations
+        linear, nearest: scipy regular grid interpolations
     FILL_VALUE: output invalid value (default NaN)
 
     Returns
@@ -165,7 +161,7 @@ def compute_tide_corrections(x, y, delta_time, DIRECTORY=None, MODEL=None,
         SCALE = 1.0/1000.0
     elif (MODEL == 'TPXO9-atlas-v2'):
         model_directory = os.path.join(DIRECTORY,'TPXO9_atlas_v2')
-        grid_file = 'grid_tpxo9_atlas_v2.nc.gz'
+        grid_file = 'grid_tpxo9_atlas_30_v2.nc.gz'
         model_files = ['h_q1_tpxo9_atlas_30_v2.nc.gz','h_o1_tpxo9_atlas_30_v2.nc.gz',
             'h_p1_tpxo9_atlas_30_v2.nc.gz','h_k1_tpxo9_atlas_30_v2.nc.gz',
             'h_n2_tpxo9_atlas_30_v2.nc.gz','h_m2_tpxo9_atlas_30_v2.nc.gz',
@@ -173,6 +169,19 @@ def compute_tide_corrections(x, y, delta_time, DIRECTORY=None, MODEL=None,
             'h_m4_tpxo9_atlas_30_v2.nc.gz','h_ms4_tpxo9_atlas_30_v2.nc.gz',
             'h_mn4_tpxo9_atlas_30_v2.nc.gz','h_2n2_tpxo9_atlas_30_v2.nc.gz']
         model_format = 'netcdf'
+        model_type = 'z'
+        SCALE = 1.0/1000.0
+    elif (MODEL == 'TPXO9-atlas-v3'):
+        model_directory = os.path.join(DIRECTORY,'TPXO9_atlas_v3')
+        grid_file = 'grid_tpxo9_atlas_30_v3.nc.gz'
+        model_files = ['h_q1_tpxo9_atlas_30_v3.nc.gz','h_o1_tpxo9_atlas_30_v3.nc.gz',
+            'h_p1_tpxo9_atlas_30_v3.nc.gz','h_k1_tpxo9_atlas_30_v3.nc.gz',
+            'h_n2_tpxo9_atlas_30_v3.nc.gz','h_m2_tpxo9_atlas_30_v3.nc.gz',
+            'h_s2_tpxo9_atlas_30_v3.nc.gz','h_k2_tpxo9_atlas_30_v3.nc.gz',
+            'h_m4_tpxo9_atlas_30_v3.nc.gz','h_ms4_tpxo9_atlas_30_v3.nc.gz',
+            'h_mn4_tpxo9_atlas_30_v3.nc.gz','h_2n2_tpxo9_atlas_30_v3.nc.gz',
+            'h_mf_tpxo9_atlas_30_v3.nc.gz','h_mm_tpxo9_atlas_30_v3.nc.gz']
+        model_format = 'netcdf'
         TYPE = 'z'
         SCALE = 1.0/1000.0
     elif (MODEL == 'TPXO9.1'):
@@ -296,12 +305,13 @@ def compute_tide_corrections(x, y, delta_time, DIRECTORY=None, MODEL=None,
         raise Exception("Unlisted tide model")
 
     #-- converting x,y from EPSG to latitude/longitude
-    proj1 = pyproj.Proj("+init=EPSG:{0:d}".format(EPSG))
-    proj2 = pyproj.Proj("+init=EPSG:{0:d}".format(4326))
-    lon,lat = pyproj.transform(proj1, proj2, x.flatten(), y.flatten())
+    crs1 = pyproj.CRS.from_string("epsg:{0:d}".format(EPSG))
+    crs2 = pyproj.CRS.from_string("epsg:{0:d}".format(4326))
+    transformer = pyproj.Transformer.from_crs(crs1, crs2, always_xy=True)
+    lon,lat = transformer.transform(x.flatten(), y.flatten())
 
-    #-- convert delta time from point to array
-    delta_time = point_to_array(delta_time)
+    #-- assert delta time is an array
+    delta_time = np.atleast_1d(delta_time)
     #-- calculate leap seconds if specified
     if (TIME.upper() == 'GPS'):
         GPS_Epoch_Time = pyTMD.time.convert_delta_time(0, epoch1=EPOCH,
@@ -310,7 +320,7 @@ def compute_tide_corrections(x, y, delta_time, DIRECTORY=None, MODEL=None,
             epoch2=(1980,1,6,0,0,0), scale=1.0)
         #-- calculate difference in leap seconds from start of epoch
         leap_seconds = pyTMD.time.count_leap_seconds(GPS_Time) - \
-            pyTMD.time.count_leap_seconds(point_to_array(GPS_Epoch_Time))
+            pyTMD.time.count_leap_seconds(np.atleast_1d(GPS_Epoch_Time))
     elif (TIME.upper() == 'TAI'):
         #-- TAI time is ahead of GPS time by 19 seconds
         GPS_Epoch_Time = pyTMD.time.convert_delta_time(-19.0, epoch1=EPOCH,
@@ -319,7 +329,7 @@ def compute_tide_corrections(x, y, delta_time, DIRECTORY=None, MODEL=None,
             epoch2=(1980,1,6,0,0,0), scale=1.0)
         #-- calculate difference in leap seconds from start of epoch
         leap_seconds = pyTMD.time.count_leap_seconds(GPS_Time) - \
-            pyTMD.time.count_leap_seconds(point_to_array(GPS_Epoch_Time))
+            pyTMD.time.count_leap_seconds(np.atleast_1d(GPS_Epoch_Time))
     else:
         leap_seconds = 0.0
 
@@ -345,7 +355,9 @@ def compute_tide_corrections(x, y, delta_time, DIRECTORY=None, MODEL=None,
     elif (model_format == 'FES'):
         amp,ph = extract_FES_constants(lon, lat, model_directory, model_files,
             TYPE=model_type, VERSION=MODEL, METHOD=METHOD, SCALE=SCALE)
-        deltat = np.zeros_like(t)
+        #-- interpolate delta times from calendar dates to tide time
+        delta_file = pyTMD.utilities.get_data_path(['data','merged_deltat.data'])
+        deltat = calc_delta_time(delta_file, t)
 
     #-- calculate complex phase in radians for Euler's
     cph = -1j*ph*np.pi/180.0
