@@ -76,7 +76,7 @@ def convert_julian(JD, ASTYPE=None, FORMAT='dict'):
 
     #-- convert to array if only a single value was imported
     if (np.ndim(JD) == 0):
-        JD = np.array([JD])
+        JD = np.atleast_1d(JD)
         SINGLE_VALUE = True
     else:
         SINGLE_VALUE = False
