@@ -5,10 +5,10 @@ Calculates the nodal corrections for tidal constituents
 Modification of ARGUMENTS fortran subroutine by Richard Ray 03/1999
 
 CALLING SEQUENCE:
-    pu,pf,G = load_nodal_corrections(time,constituents)
+    pu,pf,G = load_nodal_corrections(MJD,constituents)
 
 INPUTS:
-    time: modified julian day of input date
+    MJD: Modified Julian Day of input date
     constituents: tidal constituent IDs
 
 OUTPUTS:
@@ -35,6 +35,8 @@ REFERENCES:
         time series", Advances in Water Resources, 12, (1989).
 
 UPDATE HISTORY:
+    Updated 08/2020: change time variable names to not overwrite functions
+        update nodal corrections for FES models
     Updated 07/2020: added function docstrings.  add shallow water constituents
     Updated 09/2019: added netcdf option to CORRECTIONS option
     Updated 08/2018: added correction option ATLAS for localized OTIS solutions
@@ -46,13 +48,13 @@ UPDATE HISTORY:
 import numpy as np
 from pyTMD.calc_astrol_longitudes import calc_astrol_longitudes
 
-def load_nodal_corrections(time,constituents,DELTAT=0.0,CORRECTIONS='OTIS'):
+def load_nodal_corrections(MJD,constituents,DELTAT=0.0,CORRECTIONS='OTIS'):
     """
     Calculates the nodal corrections for tidal constituents
 
     Arguments
     ---------
-    time: modified julian day of input date
+    MJD: modified julian day of input date
     constituents: tidal constituent IDs
 
     Keyword arguments
@@ -77,14 +79,14 @@ def load_nodal_corrections(time,constituents,DELTAT=0.0,CORRECTIONS='OTIS'):
     #-- degrees to radians
     dtr = np.pi/180.0
 
-    #-- set function for astrological longitudes
+    #-- set function for astronomical longitudes
     ASTRO5 = True if CORRECTIONS in ('GOT','FES') else False
     #-- convert from Modified Julian Dates into Ephemeris Time
-    s,h,p,omega,pp = calc_astrol_longitudes(time+DELTAT, ASTRO5=ASTRO5)
-    hour = (time % 1)*24.0
+    s,h,p,omega,pp = calc_astrol_longitudes(MJD+DELTAT, ASTRO5=ASTRO5)
+    hour = (MJD % 1)*24.0
     t1 = 15.0*hour
     t2 = 30.0*hour
-    nt = 1 if (np.ndim(time) == 0) else len(time)
+    nt = len(np.atleast_1d(MJD))
 
     #-- Determine equilibrium arguments
     arg = np.zeros((nt,60))
@@ -165,7 +167,7 @@ def load_nodal_corrections(time,constituents,DELTAT=0.0,CORRECTIONS='OTIS'):
     f = np.zeros((nt,60))
     u = np.zeros((nt,60))
     #-- determine nodal corrections f and u for each model type
-    if CORRECTIONS in ('OTIS','ATLAS','netcdf','FES'):
+    if CORRECTIONS in ('OTIS','ATLAS','netcdf'):
         f[:,0] = 1.0 #-- Sa
         f[:,1] = 1.0 #-- Ssa
         f[:,2] = 1.0 - 0.130*cosn #-- Mm
@@ -187,8 +189,8 @@ def load_nodal_corrections(time,constituents,DELTAT=0.0,CORRECTIONS='OTIS'):
         # Mtmp1 = 2.0*np.cos(p*dtr) + 0.4*np.cos((p-omega)*dtr)
         # Mtmp2 = np.sin(p*dtr) + 0.2*np.sin((p-omega)*dtr)
         #-- Ray's
-        Mtmp1  = 1.36*np.cos(p*dtr) + 0.267*np.cos((p-omega)*dtr)
-        Mtmp2  = 0.64*np.sin(p*dtr) + 0.135*np.sin((p-omega)*dtr)
+        Mtmp1 = 1.36*np.cos(p*dtr) + 0.267*np.cos((p-omega)*dtr)
+        Mtmp2 = 0.64*np.sin(p*dtr) + 0.135*np.sin((p-omega)*dtr)
         f[:,13] = np.sqrt(Mtmp1**2 + Mtmp2**2) #-- M1
         f[:,14] = np.sqrt((1.0+0.221*cosn)**2+(0.221*sinn)**2) #-- chi1
         f[:,15] = 1.0 #-- pi1
@@ -214,9 +216,9 @@ def load_nodal_corrections(time,constituents,DELTAT=0.0,CORRECTIONS='OTIS'):
         f[:,29] = f[:,24] #-- M2
         f[:,30] = 1.0 #-- M2b
         f[:,31] = 1.0 #-- lambda2
-        temp1 = 1.0 - 0.25*np.cos(2*p*dtr) - 0.11*np.cos((2.0*p-omega)*dtr) - 0.04*cosn
-        temp2 = 0.25*np.sin(2*p*dtr) + 0.11*np.sin((2.0*p-omega)*dtr) + 0.04*sinn
-        f[:,32] = np.sqrt(temp1**2 + temp2**2) #-- L2
+        Ltmp1 = 1.0 - 0.25*np.cos(2*p*dtr) - 0.11*np.cos((2.0*p-omega)*dtr) - 0.04*cosn
+        Ltmp2 = 0.25*np.sin(2*p*dtr) + 0.11*np.sin((2.0*p-omega)*dtr) + 0.04*sinn
+        f[:,32] = np.sqrt(Ltmp1**2 + Ltmp2**2) #-- L2
         f[:,33] = 1.0 #-- T2
         f[:,34] = 1.0 #-- S2
         f[:,35] = 1.0 #-- R2
@@ -289,7 +291,7 @@ def load_nodal_corrections(time,constituents,DELTAT=0.0,CORRECTIONS='OTIS'):
         u[:,29] = u[:,24] #-- M2
         u[:,30] = 0.0 #-- M2b
         u[:,31] = 0.0 #-- lambda2
-        u[:,32] = np.arctan(-temp2/temp1)/dtr #-- L2
+        u[:,32] = np.arctan(-Ltmp2/Ltmp1)/dtr #-- L2
         u[:,33] = 0.0 #-- T2
         u[:,34] = 0.0 #-- S2
         u[:,35] = 0.0 #-- R2
@@ -312,13 +314,150 @@ def load_nodal_corrections(time,constituents,DELTAT=0.0,CORRECTIONS='OTIS'):
         u[:,50] = 0.0 #-- S6
         u[:,51] = 0.0 #-- S7
         u[:,52] = 0.0 #-- S8
+        #-- mean sea level
+        u[:,59] = 0.0 #-- Z0
+
+    elif CORRECTIONS in ('FES',):
+        #-- additional astronomical terms for FES models
+        II = np.arccos(0.913694997 - 0.035692561*np.cos(omega*dtr))
+        at1 = np.arctan(1.01883*np.tan(omega*dtr/2.0))
+        at2 = np.arctan(0.64412*np.tan(omega*dtr/2.0))
+        xi = -at1 - at2 + omega*dtr
+        xi[xi > np.pi] -= 2.0*np.pi
+        nu = at1 - at2
+        I2 = np.tan(II/2.0)
+        Ra1 = np.sqrt(1.0 - 12.0*(I2**2)*np.cos(2.0*(p - xi)) + 36.0*(I2**4))
+        P2 = np.sin(2.0*(p - xi))
+        Q2 = 1.0/(6.0*(I2**2)) - np.cos(2.0*(p - xi))
+        R = np.arctan(P2/Q2)
+        P_prime = np.sin(2.0*II)*np.sin(nu)
+        Q_prime = np.sin(2.0*II)*np.cos(nu) + 0.3347
+        nu_prime = np.arctan(P_prime/Q_prime)
+        P_sec = (np.sin(II)**2)*np.sin(2.0*nu)
+        Q_sec = (np.sin(II)**2)*np.cos(2.0*nu) + 0.0727
+        nu_sec = 0.5*np.arctan(P_sec/Q_sec)
+
+        f[:,0] = 1.0 #-- Sa
+        f[:,1] = 1.0 #-- Ssa
+        f[:,2] = (2.0/3.0 - np.power(np.sin(II),2.0))/0.5021 #-- Mm
+        f[:,3] = 1.0 #-- MSf
+        f[:,4] = np.power(np.sin(II),2.0)/0.1578  #-- Mf
+        f[:,7] = np.sin(II)*(np.cos(II/2.0)**2)/0.38 #-- 2Q1
+        f[:,8] = f[:,7] #-- sigma1
+        f[:,9] = f[:,7] #-- q1
+        f[:,10] = f[:,7] #-- rho1
+        f[:,11] = f[:,7] #-- O1
+        #-- Ray's
+        Mtmp1 = 1.36*np.cos(p*dtr) + 0.267*np.cos((p-omega)*dtr)
+        Mtmp2 = 0.64*np.sin(p*dtr) + 0.135*np.sin((p-omega)*dtr)
+        f[:,13] = np.sqrt(Mtmp1**2 + Mtmp2**2) #-- M1
+        f[:,14] = np.sin(2.0*II) / 0.7214 #-- chi1
+        f[:,15] = 1.0 #-- pi1
+        f[:,16] = 1.0 #-- P1
+        f[:,17] = 1.0 #-- S1
+        temp1 = 0.8965*np.power(np.sin(2.0*II),2.0)
+        temp2 = 0.6001*np.sin(2.0*II)*np.cos(nu)
+        f[:,18] = np.sqrt(temp1 + temp1 + 0.1006) #-- K1
+        f[:,19] = 1.0 #-- psi1
+        f[:,20] = 1.0 #-- phi1
+        f[:,21] = f[:,14] #-- theta1
+        f[:,22] = f[:,14] #-- J1
+        f[:,23] = np.sin(II)*np.power(np.sin(II/2.0),2.0)/0.01640 #-- OO1
+        f[:,24] = np.power(np.cos(II/2.0),4.0)/0.9154 #-- 2N2
+        f[:,25] = f[:,24] #-- mu2
+        f[:,26] = f[:,24] #-- N2
+        f[:,27] = f[:,24] #-- nu2
+        f[:,28] = 1.0 #-- M2a
+        f[:,29] = f[:,24] #-- M2
+        f[:,30] = 1.0 #-- M2b
+        f[:,31] = f[:,29] #-- lambda2
+        f[:,32] = f[:,29]*Ra1 #-- L2
+        f[:,33] = 1.0 #-- T2
+        f[:,34] = 1.0 #-- S2
+        f[:,35] = 1.0 #-- R2
+        temp1 = 19.0444 * np.power(np.sin(II),4.0)
+        temp2 = 2.7702 * np.power(np.sin(II),2.0) * np.cos(2.0*nu)
+        f[:,36] = np.sqrt(temp1 + temp2 + 0.0981) #-- K2
+        f[:,37] = np.power(np.sin(II),2.0)/0.1565 #-- eta2
+        f[:,38] = f[:,29]**2 #-- MNS2
+        f[:,39] = f[:,29] #-- 2SM2
+        f[:,40] = np.power(np.cos(II/2.0), 6.0) / 0.8758 #-- M3
+        f[:,41] = f[:,18]*f[:,29] #-- MK3
+        f[:,42] = 1.0 #-- S3
+        f[:,43] = f[:,29]**2 #-- MN4
+        f[:,44] = f[:,43] #-- M4
+        f[:,45] = f[:,29] #-- MS4
+        f[:,46] = f[:,29]*f[:,36] #-- MK4
+        f[:,47] = 1.0 #-- S4
+        f[:,48] = 1.0 #-- S5
+        f[:,49] = f[:,29]**3 #-- M6
+        f[:,50] = 1.0 #-- S6
+        f[:,51] = 1.0 #-- S7
+        f[:,52] = 1.0 #-- S8
         #-- shallow water constituents
-        u[:,53] = 4.0*u[:,29] #-- m8
-        u[:,54] = u[:,29] + u[:,36] #-- mks2
+        f[:,53] = f[:,29]**4 #-- m8
+        f[:,54] = f[:,29]*f[:,36] #-- mks2
+        f[:,55] = f[:,4] #-- msqm
+        f[:,56] = f[:,4] #-- mtm
+        f[:,57] = f[:,29]**2 #-- n4
+        f[:,58] = f[:,29] #-- eps2
+        #-- mean sea level
+        f[:,59] = 1.0 #-- Z0
+
+        u[:,0] = 0.0 #-- Sa
+        u[:,1] = 0.0 #-- Ssa
+        u[:,2] = 0.0 #-- Mm
+        u[:,3] = (2.0*xi - 2.0*nu)/dtr #-- MSf
+        u[:,4] = -2.0*xi/dtr #-- Mf
+        u[:,7] = (2.0*xi - nu)/dtr #-- 2Q1
+        u[:,8] = u[:,7] #-- sigma1
+        u[:,9] = u[:,7] #-- q1
+        u[:,10] = u[:,7] #-- rho1
+        u[:,11] = u[:,7] #-- O1
+        u[:,13] = np.arctan2(Mtmp2,Mtmp1)/dtr #-- M1
+        u[:,14] = -nu/dtr #-- chi1
+        u[:,15] = 0.0 #-- pi1
+        u[:,16] = 0.0 #-- P1
+        u[:,17] = 0.0 #-- S1
+        u[:,18] = -nu_prime/dtr #-- K1
+        u[:,19] = 0.0 #-- psi1
+        u[:,20] = 0.0 #-- phi1
+        u[:,21] = -nu/dtr #-- theta1
+        u[:,22] = u[:,21] #-- J1
+        u[:,23] = (-2.0*xi - nu)/dtr #-- OO1
+        u[:,24] = (2.0*xi - 2.0*nu)/dtr #-- 2N2
+        u[:,25] = u[:,24] #-- mu2
+        u[:,26] = u[:,24] #-- N2
+        u[:,27] = u[:,24] #-- nu2
+        u[:,29] = u[:,24] #-- M2
+        u[:,31] = (2.0*xi - 2.0*nu)/dtr #-- lambda2
+        u[:,32] = (2.0*xi - 2.0*nu - R)/dtr #-- L2
+        u[:,33] = 0.0 #-- T2
+        u[:,34] = 0.0 #-- S2
+        u[:,35] = 0.0 #-- R2
+        u[:,36] = -2.0*nu_sec/dtr #-- K2
+        u[:,37] = -2.0*nu/dtr #-- eta2
+        u[:,38] = (4.0*xi - 4.0*nu)/dtr #-- mns2
+        u[:,39] = (2.0*xi - 2.0*nu)/dtr #-- 2SM2
+        u[:,40] = (3.0*xi - 3.0*nu)/dtr #-- M3
+        u[:,41] = (2.0*xi - 2.0*nu - 2.0*nu_prime)/dtr #-- MK3
+        u[:,42] = 0.0 #-- S3
+        u[:,43] = (4.0*xi - 4.0*nu)/dtr #-- MN4
+        u[:,44] = (4.0*xi - 4.0*nu)/dtr #-- M4
+        u[:,45] = (2.0*xi - 2.0*nu)/dtr  #-- MS4
+        u[:,46] = (2.0*xi - 2.0*nu - 2.0*nu_sec)/dtr #-- MK4
+        u[:,47] = 0.0 #-- S4
+        u[:,48] = 0.0 #-- S5
+        u[:,49] = (6.0*xi - 6.0*nu)/dtr #-- M6
+        u[:,50] = 0.0 #-- S6
+        u[:,51] = 0.0 #-- S7
+        u[:,52] = 0.0 #-- S8
+        #-- shallow water constituents
+        u[:,53] = (8.0*xi - 8.0*nu)/dtr #-- m8
+        u[:,54] = (2.0*xi - 2.0*nu - 2.0*nu_sec)/dtr #-- mks2
         u[:,55] = u[:,4] #-- msqm
         u[:,56] = u[:,4] #-- mtm
-        u[:,57] = 2.0*u[:,29] #-- n4
-        u[:,57] = 2.0*u[:,29] #-- MN4
+        u[:,57] = (4.0*xi - 4.0*nu)/dtr #-- n4
         u[:,58] = u[:,29] #-- eps2
         #-- mean sea level
         u[:,59] = 0.0 #-- Z0
