@@ -1,13 +1,13 @@
 #!/usr/bin/env python
 u"""
-predict_tide_drift.py (07/2020)
+predict_tide_drift.py (09/2020)
 Predict tides at multiple times and locations using harmonic constants
 
 CALLING SEQUENCE:
-    ht = predict_tide_drift(time,hc,con)
+    ht = predict_tide_drift(t,hc,con)
 
 INPUTS:
-    time: days relative to Jan 1, 1992 (48622mjd)
+    t: days relative to Jan 1, 1992 (48622mjd)
     hc: harmonic constant vector (complex)
     constituents: tidal constituent IDs
 
@@ -28,6 +28,8 @@ PROGRAM DEPENDENCIES:
     load_nodal_corrections.py: loads nodal corrections for tidal constituents
 
 UPDATE HISTORY:
+    Updated 09/2020: append output mask over each constituent
+    Updated 08/2020: change time variable names to not overwrite functions
     Updated 07/2020: added function docstrings
     Updated 11/2019: output as numpy masked arrays instead of nan-filled arrays
     Updated 09/2019: added netcdf option to CORRECTIONS option
@@ -39,13 +41,13 @@ import numpy as np
 from pyTMD.load_constituent import load_constituent
 from pyTMD.load_nodal_corrections import load_nodal_corrections
 
-def predict_tide_drift(time,hc,constituents,DELTAT=0.0,CORRECTIONS='OTIS'):
+def predict_tide_drift(t,hc,constituents,DELTAT=0.0,CORRECTIONS='OTIS'):
     """
     Predict tides at multiple times and locations using harmonic constants
 
     Arguments
     ---------
-    time: days relative to 1992-01-01T00:00:00
+    t: days relative to 1992-01-01T00:00:00
     hc: harmonic constant vector (complex)
     constituents: tidal constituent IDs
 
@@ -59,22 +61,26 @@ def predict_tide_drift(time,hc,constituents,DELTAT=0.0,CORRECTIONS='OTIS'):
     ht: tidal time series reconstructed using the nodal corrections
     """
 
-    nt = len(time)
+    nt = len(t)
     #-- load the nodal corrections
-    pu,pf,G = load_nodal_corrections(time + 48622.0, constituents,
+    #-- convert time to Modified Julian Days (MJD)
+    pu,pf,G = load_nodal_corrections(t + 48622.0, constituents,
         DELTAT=DELTAT, CORRECTIONS=CORRECTIONS)
     #-- allocate for output time series
     ht = np.ma.zeros((nt))
+    ht.mask = np.zeros((nt),dtype=np.bool)
     #-- for each constituent
     for k,c in enumerate(constituents):
         if CORRECTIONS in ('OTIS','ATLAS','netcdf'):
             #-- load parameters for each constituent
             amp,ph,omega,alpha,species = load_constituent(c)
             #-- add component for constituent to output tidal elevation
-            th = omega*time*86400.0 + ph + pu[:,k]
+            th = omega*t*86400.0 + ph + pu[:,k]
         elif CORRECTIONS in ('GOT','FES'):
             th = G[:,k]*np.pi/180.0 + pu[:,k]
         #-- sum over all tides
-        ht += pf[:,k]*hc.real[:,k]*np.cos(th) - pf[:,k]*hc.imag[:,k]*np.sin(th)
+        ht.data[:] += pf[:,k]*hc.real[:,k]*np.cos(th) - \
+            pf[:,k]*hc.imag[:,k]*np.sin(th)
+        ht.mask[:] |= (hc.real.mask[:,k] | hc.imag.mask[:,k])
     #-- return tides
     return ht
