@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 u"""
-read_GOT_model.py (07/2020)
+read_GOT_model.py (09/2020)
 Reads files for Richard Ray's Global Ocean Tide (GOT) models and makes initial
     calculations to run the tide program
 Includes functions to extract tidal harmonic constants out of a tidal model for
@@ -16,7 +16,7 @@ OPTIONS:
     METHOD: interpolation method
         bilinear: quick bilinear interpolation
         spline: scipy bivariate spline interpolation
-        linear, cubic, nearest: scipy griddata interpolations
+        linear, nearest: scipy regular grid interpolations
     GZIP: input files are compressed
     SCALE: scaling factor for converting to output units
 
@@ -35,6 +35,9 @@ PROGRAM DEPENDENCIES:
     bilinear_interp.py: bilinear interpolation of data to specified coordinates
 
 UPDATE HISTORY:
+    Updated 09/2020: set bounds error to false for regular grid interpolations
+        adjust dimensions of input coordinates to be iterable
+    Updated 08/2020: replaced griddata with scipy regular grid interpolators
     Updated 07/2020: added function docstrings. separate bilinear interpolation
         update griddata interpolation. add option GZIP for compression
     Updated 06/2020: use argmin and argmax in bilinear interpolation
@@ -74,7 +77,7 @@ def extract_GOT_constants(ilon, ilat, directory, model_files,
     METHOD: interpolation method
         bilinear: quick bilinear interpolation
         spline: scipy bivariate spline interpolation
-        linear, cubic, nearest: scipy griddata interpolations
+        linear, nearest: scipy regular grid interpolations
     GZIP: input files are compressed
     SCALE: scaling factor for converting to output units
 
@@ -83,6 +86,10 @@ def extract_GOT_constants(ilon, ilat, directory, model_files,
     amplitude: amplitudes of tidal constituents
     phase: phases of tidal constituents
     """
+
+    #-- adjust dimensions of input coordinates to be iterable
+    ilon = np.atleast_1d(ilon)
+    ilat = np.atleast_1d(ilat)
     #-- adjust longitudinal convention of input latitude and longitude
     #-- to fit tide model convention
     if (np.min(ilon) < 0.0):
@@ -118,27 +125,33 @@ def extract_GOT_constants(ilon, ilat, directory, model_files,
             #-- use quick bilinear to interpolate values
             hci.data[:] = bilinear_interp(lon,lat,hc,ilon,ilat,dtype=hc.dtype)
             #-- replace nan values with fill_value
-            hci.mask[:] = np.isnan(hci.data)
+            hci.mask[:] |= np.isnan(hci.data)
             hci.data[hci.mask] = hci.fill_value
         elif (METHOD == 'spline'):
             #-- interpolate complex form of the constituent with scipy
-            f1=scipy.interpolate.RectBivariateSpline(lon,lat,hc.data.real.T,kx=1,ky=1)
-            f2=scipy.interpolate.RectBivariateSpline(lon,lat,hc.data.imag.T,kx=1,ky=1)
-            f3=scipy.interpolate.RectBivariateSpline(lon,lat,hc.mask.T,kx=1,ky=1)
+            f1=scipy.interpolate.RectBivariateSpline(lon,lat,
+                hc.data.real.T,kx=1,ky=1)
+            f2=scipy.interpolate.RectBivariateSpline(lon,lat,
+                hc.data.imag.T,kx=1,ky=1)
+            f3=scipy.interpolate.RectBivariateSpline(lon,lat,
+                hc.mask.T,kx=1,ky=1)
             hci.data.real[:] = f1.ev(ilon,ilat)
             hci.data.imag[:] = f2.ev(ilon,ilat)
             hci.mask[:] = f3.ev(ilon,ilat).astype(np.bool)
+            #-- replace invalid values with fill_value
+            hci.data[hci.mask] = hci.fill_value
         else:
-            #-- create mesh grids of latitude and longitude
-            X,Y = np.meshgrid(lon,lat)
-            interp_points = np.c_[X.flatten(),Y.flatten()]
-            #-- replace invalid values with nan
-            hc[hc.mask] = np.nan
-            #-- interpolate complex form of the constituent with scipy
-            hci.data[:] = scipy.interpolate.griddata(interp_points,
-                hc.flatten(), np.c_[ilon,ilat], method=METHOD)
-            #-- mask invalid values
-            hci.mask[:] = np.isnan(hci.data[:])
+            #-- use scipy regular grid to interpolate values for a given method
+            r1 = scipy.interpolate.RegularGridInterpolator((lat,lon),
+                hc.data, method=METHOD, bounds_error=False,
+                fill_value=hci.fill_value)
+            r2 = scipy.interpolate.RegularGridInterpolator((lat,lon),
+                hc.mask, method=METHOD, bounds_error=False, fill_value=1)
+            hci.data[:] = r1.__call__(np.c_[ilat,ilon])
+            hci.mask[:] = np.ceil(r2.__call__(np.c_[ilat,ilon])).astype(np.bool)
+            #-- replace invalid values with fill_value
+            hci.mask[:] |= (hci.data == hci.fill_value)
+            hci.data[hci.mask] = hci.fill_value
         #-- convert amplitude from input units to meters
         amplitude.data[:,i] = np.abs(hci)*SCALE
         amplitude.mask[:,i] = np.copy(hci.mask)
