@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 u"""
-read_netcdf_model.py (07/2020)
+read_netcdf_model.py (09/2020)
 Reads files for a tidal model and makes initial calculations to run tide program
 Includes functions to extract tidal harmonic constants from OTIS tide models for
     given locations
@@ -28,7 +28,7 @@ OPTIONS:
     METHOD: interpolation method
         bilinear: quick bilinear interpolation
         spline: scipy bivariate spline interpolation
-        linear, cubic, nearest: scipy griddata interpolations
+        linear, nearest: scipy regular grid interpolations
     GZIP: input netCDF4 files are compressed
     SCALE: scaling factor for converting to output units
 
@@ -51,6 +51,10 @@ PROGRAM DEPENDENCIES:
     bilinear_interp.py: bilinear interpolation of data to specified coordinates
 
 UPDATE HISTORY:
+    Updated 09/2020: set bounds error to false for regular grid interpolations
+        adjust dimensions of input coordinates to be iterable
+        reduce number of interpolations by copying bathmetry mask to variables
+    Updated 08/2020: replaced griddata with scipy regular grid interpolators
     Updated 07/2020: added function docstrings. separate bilinear interpolation
         changed TYPE variable to keyword argument. update griddata interpolation
     Updated 06/2020: use argmin and argmax in bilinear interpolation
@@ -90,7 +94,7 @@ def extract_netcdf_constants(ilon, ilat, directory, grid_file, model_files,
     METHOD: interpolation method
         bilinear: quick bilinear interpolation
         spline: scipy bivariate spline interpolation
-        linear, cubic, nearest: scipy griddata interpolations
+        linear, nearest: scipy regular grid interpolations
     GZIP: input netCDF4 files are compressed
     SCALE: scaling factor for converting to output units
 
@@ -150,6 +154,9 @@ def extract_netcdf_constants(ilon, ilat, directory, grid_file, model_files,
     #-- create meshes from latitude and longitude
     gridlon,gridlat = np.meshgrid(lon,lat)
 
+    #-- adjust dimensions of input coordinates to be iterable
+    ilon = np.atleast_1d(ilon)
+    ilat = np.atleast_1d(ilat)
     #-- adjust longitudinal convention of input latitude and longitude
     #-- to fit tide model convention
     lt0, = np.nonzero(ilon < 0)
@@ -161,30 +168,33 @@ def extract_netcdf_constants(ilon, ilat, directory, grid_file, model_files,
     D = np.ma.zeros((npts))
     D.mask = np.zeros((npts),dtype=np.bool)
     if (METHOD == 'bilinear'):
+        #-- replace invalid values with nan
+        bathymetry[bathymetry.mask] = np.nan
         #-- use quick bilinear to interpolate values
-        D.data[:] = bilinear_interp(lon,lat,bathymetry.data,ilon,ilat)
-        D.mask[:] = bilinear_interp(lon,lat,bathymetry.mask,ilon,ilat)
+        D.data[:] = bilinear_interp(lon,lat,bathymetry,ilon,ilat)
+        #-- replace nan values with fill_value
+        D.mask[:] = np.isnan(D.data)
+        D.data[D.mask] = D.fill_value
     elif (METHOD == 'spline'):
         #-- use scipy bivariate splines to interpolate values
-        f1=scipy.interpolate.RectBivariateSpline(lon,lat,
+        f1 = scipy.interpolate.RectBivariateSpline(lon,lat,
             bathymetry.data.T,kx=1,ky=1)
-        f2=scipy.interpolate.RectBivariateSpline(lon,lat,
+        f2 = scipy.interpolate.RectBivariateSpline(lon,lat,
             bathymetry.mask.T,kx=1,ky=1)
         D.data[:] = f1.ev(ilon,ilat)
-        D.mask[:] = f2.ev(ilon,ilat).astype(np.bool)
+        D.mask[:] = np.ceil(f2.ev(ilon,ilat).astype(np.bool))
     else:
-        #-- use scipy griddata to interpolate values
-        interp_points = np.c_[lon.flatten(),lat.flatten()]
-        D.data[:] = scipy.interpolate.griddata(interp_points,
-            bathymetry.data.flatten(), np.c_[ilon,ilat],
-            method=METHOD)
-        D.mask[:] = scipy.interpolate.griddata(interp_points,
-            bathymetry.mask.flatten(), np.c_[ilon,ilat],
-            method=METHOD).astype(np.bool)
+        #-- use scipy regular grid to interpolate values for a given method
+        r1 = scipy.interpolate.RegularGridInterpolator((lat,lon),
+            bathymetry.data, method=METHOD, bounds_error=False)
+        r2 = scipy.interpolate.RegularGridInterpolator((lat,lon),
+            bathymetry.mask, method=METHOD, bounds_error=False, fill_value=1)
+        D.data[:] = r1.__call__(np.c_[ilat,ilon])
+        D.mask[:] = np.ceil(r2.__call__(np.c_[ilat,ilon])).astype(np.bool)
 
     #-- u and v are velocities in cm/s
     if TYPE in ('v','u'):
-        unit_conv = (D.data*100.0)
+        unit_conv = (D.data/100.0)
     #-- U and V are transports in m^2/s
     elif TYPE in ('V','U'):
         unit_conv = 1.0
@@ -212,28 +222,27 @@ def extract_netcdf_constants(ilon, ilat, directory, grid_file, model_files,
             z1.mask = np.zeros((npts),dtype=np.bool)
             if (METHOD == 'bilinear'):
                 z1.data[:] = bilinear_interp(lon,lat,z,ilon,ilat,dtype=z.dtype)
-                #-- mask zero values
-                z1.mask[z1.data == 0] = True
+                #-- mask invalid values
+                z1.mask[:] |= np.copy(D.mask)
                 z1.data[z1.mask] = z1.fill_value
             elif (METHOD == 'spline'):
                 f1 = scipy.interpolate.RectBivariateSpline(lon,lat,
                     z.data.real.T,kx=1,ky=1)
                 f2 = scipy.interpolate.RectBivariateSpline(lon,lat,
                     z.data.imag.T,kx=1,ky=1)
-                f3 = scipy.interpolate.RectBivariateSpline(lon,lat,
-                    z.mask.T,kx=1,ky=1)
                 z1.data.real = f1.ev(ilon,ilat)
                 z1.data.imag = f2.ev(ilon,ilat)
-                z1.mask = f3.ev(ilon,ilat).astype(np.bool)
                 #-- mask invalid values
+                z1.mask[:] |= np.copy(D.mask)
                 z1.data[z1.mask] = z1.fill_value
             else:
-                #-- replace zero values with nan
-                z[z==0] = np.nan
-                #-- use scipy griddata to interpolate values
-                z1.data = scipy.interpolate.griddata(interp_points,
-                    z.flatten(), np.c_[ilon,ilat], method=METHOD)
-                z1.mask[np.isnan(z1.data)] = True
+                #-- use scipy regular grid to interpolate values
+                r1 = scipy.interpolate.RegularGridInterpolator((lat,lon),
+                    z.data, method=METHOD, bounds_error=False,
+                    fill_value=z1.fill_value)
+                z1.data[:] = r1.__call__(np.c_[ilat,ilon])
+                #-- mask invalid values
+                z1.mask[:] |= np.copy(D.mask)
                 z1.data[z1.mask] = z1.fill_value
             #-- amplitude and phase of the constituent
             ampl[:,i] = np.abs(z1)
@@ -245,33 +254,32 @@ def extract_netcdf_constants(ilon, ilat, directory, grid_file, model_files,
             constituents.append(con)
             #-- replace original values with extend matrices
             tr = extend_matrix(tr)
-            #-- interpolate values
+            #-- interpolate amplitude and phase of the constituent
             tr1 = np.ma.zeros((npts),dtype=tr.dtype)
             tr1.mask = np.zeros((npts),dtype=np.bool)
             if (METHOD == 'bilinear'):
                 tr1.data[:]=bilinear_interp(lon,lat,tr,ilon,ilat,dtype=tr.dtype)
-                #-- mask zero values
-                tr1.mask[tr1.data == 0] = True
+                #-- mask invalid values
+                tr1.mask[:] |= np.copy(D.mask)
                 tr1.data[tr1.mask] = tr1.fill_value
             elif (METHOD == 'spline'):
                 f1 = scipy.interpolate.RectBivariateSpline(lon,lat,
                     tr.data.real.T,kx=1,ky=1)
                 f2 = scipy.interpolate.RectBivariateSpline(lon,lat,
                     tr.data.imag.T,kx=1,ky=1)
-                f3 = scipy.interpolate.RectBivariateSpline(lon,lat,
-                    tr.mask.T,kx=1,ky=1)
                 tr1.data.real = f1.ev(ilon,ilat)
                 tr1.data.imag = f2.ev(ilon,ilat)
-                tr1.mask = f3.ev(ilon,ilat).astype(np.bool)
                 #-- mask invalid values
+                tr1.mask[:] |= np.copy(D.mask)
                 tr1.data[tr1.mask] = z1.fill_value
             else:
-                #-- replace zero values with nan
-                tr[tr==0] = np.nan
-                #-- use scipy interpolate to interpolate values
-                tr1 = scipy.interpolate.griddata(interp_points,
-                    tr.flatten(), np.c_[ilon,ilat], method=METHOD)
-                tr1.mask[np.isnan(tr1.data)] = True
+                #-- use scipy regular grid to interpolate values
+                r1 = scipy.interpolate.RegularGridInterpolator((lat,lon),
+                    tr.data, method=METHOD, bounds_error=False,
+                    fill_value=tr1.fill_value)
+                tr1.data[:] = r1.__call__(np.c_[ilat,ilon])
+                #-- mask invalid values
+                tr1.mask[:] |= np.copy(D.mask)
                 tr1.data[tr1.mask] = tr1.fill_value
             #-- convert units
             tr1 = tr1/unit_conv
