@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 u"""
-read_ocean_pole_tide.py (07/2020)
+read_ocean_pole_tide.py (08/2020)
 Reads ocean pole load tide coefficients provided by IERS
 http://maia.usno.navy.mil/conventions/2010/2010_official/chapter7/tn36_c7.pdf
 http://maia.usno.navy.mil/conventions/2010/2010_update/chapter7/icc7.pdf
@@ -9,6 +9,13 @@ IERS 0.5x0.5 map of ocean pole tide coefficients:
 ftp://maia.usno.navy.mil/conventions/2010/2010_update/chapter7/additional_info/
     opoleloadcoefcmcor.txt.gz
 
+OUTPUTS:
+    ur: radial ocean pole tide coefficients
+    un: north ocean pole tide coefficients
+    ue: east ocean pole tide coefficients
+    glon: ocean grid longitude
+    glat: ocean grid latitude
+
 PYTHON DEPENDENCIES:
     numpy: Scientific Computing Tools For Python
         https://numpy.org
@@ -22,6 +29,7 @@ REFERENCES:
         doi: 10.1007/s00190-015-0848-7
 
 UPDATE HISTORY:
+    Updated 08/2020: output north load and east load deformation components
     Updated 07/2020: added function docstrings
     Updated 12/2018: Compatibility updates for Python3
     Written 09/2017
@@ -41,7 +49,9 @@ def read_ocean_pole_tide(input_file):
 
     Returns
     -------
-    ur: ocean pole tide coefficients
+    ur: radial ocean pole tide coefficients
+    un: north ocean pole tide coefficients
+    ue: east ocean pole tide coefficients
     glon: ocean grid longitude
     glat: ocean grid latitude
     """
@@ -69,18 +79,24 @@ def read_ocean_pole_tide(input_file):
     nlat = len(glat)
     #-- allocate for output grid maps
     ur = np.zeros((nlon,nlat),dtype=np.complex128)
+    un = np.zeros((nlon,nlat),dtype=np.complex128)
+    ue = np.zeros((nlon,nlat),dtype=np.complex128)
     #-- read lines of file and add to output variables
     for i,line in enumerate(file_contents[count:]):
         ln,lt,urr,uri,unr,uni,uer,uei = np.array(line.split(), dtype='f8')
         ilon = np.int(ln/dlon)
         ilat = np.int((90.0-lt)/dlat)
         ur[ilon,ilat] = urr + 1j*uri
+        un[ilon,ilat] = unr + 1j*uni
+        ue[ilon,ilat] = uer + 1j*uei
 
     #-- extend matrix for bilinear interpolation
     glon = extend_array(glon,dlon)
     ur = extend_matrix(ur)
+    un = extend_matrix(un)
+    ue = extend_matrix(ue)
     #-- return values
-    return (ur,glon,glat)
+    return (ur,un,ue,glon,glat)
 
 #-- PURPOSE: wrapper function to extend an array
 def extend_array(input_array,step_size):
