@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 u"""
-read_tide_model.py (07/2020)
+read_tide_model.py (09/2020)
 Reads files for a tidal model and makes initial calculations to run tide program
 Includes functions to extract tidal harmonic constants from OTIS tide models for
     given locations
@@ -27,7 +27,7 @@ OPTIONS:
     METHOD: interpolation method
         bilinear: quick bilinear interpolation
         spline: scipy bivariate spline interpolation
-        linear, cubic, nearest: scipy griddata interpolations
+        linear, nearest: scipy regular grid interpolations
     GRID: binary file type to read
         ATLAS: reading a global solution with localized solutions
         OTIS: combined global solution
@@ -50,6 +50,11 @@ PROGRAM DEPENDENCIES:
     bilinear_interp.py: bilinear interpolation of data to specified coordinates
 
 UPDATE HISTORY:
+    Updated 09/2020: set bounds error to false for regular grid interpolations
+        adjust dimensions of input coordinates to be iterable
+        use masked arrays with atlas models and grids. make 2' grid with nearest
+    Updated 08/2020: check that interpolated points are within range of model
+        replaced griddata interpolation with scipy regular grid interpolators
     Updated 07/2020: added function docstrings. separate bilinear interpolation
         update griddata interpolation. changed TYPE variable to keyword argument
     Updated 06/2020: output currents as numpy masked arrays
@@ -94,7 +99,7 @@ def extract_tidal_constants(ilon, ilat, grid_file, model_file, EPSG, TYPE='z',
     METHOD: interpolation method
         bilinear: quick bilinear interpolation
         spline: scipy bivariate spline interpolation
-        linear, cubic, nearest: scipy griddata interpolations
+        linear, nearest: scipy regular grid interpolations
     GRID: binary file type to read
         ATLAS: reading a global solution with localized solutions
         OTIS: combined global solution
@@ -106,7 +111,6 @@ def extract_tidal_constants(ilon, ilat, grid_file, model_file, EPSG, TYPE='z',
     D: bathymetry of tide model
     constituents: list of model constituents
     """
-
     #-- read the OTIS-format tide grid file
     if (GRID == 'ATLAS'):
         #-- if reading a global solution with localized solutions
@@ -114,10 +118,12 @@ def extract_tidal_constants(ilon, ilat, grid_file, model_file, EPSG, TYPE='z',
         xi,yi,hz = combine_atlas_model(x0,y0,hz0,pmask,local,VARIABLE='depth')
         mz = create_atlas_mask(x0,y0,mz0,local,VARIABLE='depth')
     else:
-        #-- if reading a pure global solution
+        #-- if reading a single OTIS solution
         xi,yi,hz,mz,iob,dt = read_tide_grid(grid_file)
+    #-- adjust dimensions of input coordinates to be iterable
     #-- run wrapper function to convert coordinate systems of input lat/lon
-    x,y = convert_ll_xy(ilon,ilat,EPSG,'F')
+    x,y = convert_ll_xy(np.atleast_1d(ilon),np.atleast_1d(ilat),EPSG,'F')
+    invalid = (x < xi.min()) | (x > xi.max()) | (y < yi.min()) | (y > yi.max())
     #-- grid step size of tide model
     dx = xi[1] - xi[0]
     dy = yi[1] - yi[0]
@@ -138,22 +144,13 @@ def extract_tidal_constants(ilon, ilat, grid_file, model_file, EPSG, TYPE='z',
 
     #-- adjust longitudinal convention of input latitude and longitude
     #-- to fit tide model convention
-    xmin = np.min(x)
-    xmax = np.max(y)
-    if (xmin < xi[0]) & (EPSG == '4326'):
+    if (np.min(x) < np.min(xi)) & (EPSG == '4326'):
         lt0, = np.nonzero(x < 0)
         x[lt0] += 360.0
-    if (xmax > xi[-1]) & (EPSG == '4326'):
+    if (np.max(x) > np.max(xi)) & (EPSG == '4326'):
         gt180, = np.nonzero(x > 180)
         x[gt180] -= 360.0
 
-    #-- create meshes from latitude and longitude
-    ux = xi - dx/2.0
-    vy = yi - dy/2.0
-    X,Y = np.meshgrid(xi,yi)
-    Xu,Yu = np.meshgrid(ux,yi)
-    Xv,Yv = np.meshgrid(xi,vy)
-
     #-- masks zero values
     hz = np.ma.array(hz,mask=(hz==0))
     if (TYPE != 'z'):
@@ -190,24 +187,24 @@ def extract_tidal_constants(ilon, ilat, grid_file, model_file, EPSG, TYPE='z',
             mu1 = np.ceil(f3.ev(x,y)).astype(mu.dtype)
             mv1 = np.ceil(f4.ev(x,y)).astype(mv.dtype)
     else:
-        #-- use scipy griddata to interpolate values
-        interp_points = np.c_[X.flatten(),Y.flatten()]
-        D = scipy.interpolate.griddata(interp_points,
-            hz.flatten(), np.c_[x,y], method=METHOD)
-        mz1 = scipy.interpolate.griddata(interp_points,
-            mz.real.flatten(), np.c_[x,y], method=METHOD)
-        mz1 = np.ceil(mz1).astype(mz.dtype)
+        #-- use scipy regular grid to interpolate values for a given method
+        r1 = scipy.interpolate.RegularGridInterpolator((yi,xi),hz,
+            method=METHOD,bounds_error=False)
+        r2 = scipy.interpolate.RegularGridInterpolator((yi,xi),mz,
+            method=METHOD,bounds_error=False,fill_value=0)
+        D = r1.__call__(np.c_[y,x])
+        mz1 = np.ceil(r2.__call__(np.c_[y,x])).astype(mz.dtype)
         if (TYPE != 'z'):
-            mu1 = scipy.interpolate.griddata(interp_points,
-                mu.flatten(), np.c_[x,y], method=METHOD)
-            mv1 = scipy.interpolate.griddata(interp_points,
-                mv.flatten(), np.c_[x,y], method=METHOD)
-            mu1 = np.ceil(mu1).astype(mu.dtype)
-            mv1 = np.ceil(mv1).astype(mv.dtype)
+            r3 = scipy.interpolate.RegularGridInterpolator((yi,xi),mu,
+                method=METHOD,bounds_error=False,fill_value=0)
+            r4 = scipy.interpolate.RegularGridInterpolator((yi,xi),mv,
+                method=METHOD,bounds_error=False,fill_value=0)
+            mu1 = np.ceil(r3.__call__(np.c_[y,x])).astype(mu.dtype)
+            mv1 = np.ceil(r4.__call__(np.c_[y,x])).astype(mv.dtype)
 
     #-- u and v are velocities in cm/s
     if TYPE in ('v','u'):
-        unit_conv = (D*100.0)
+        unit_conv = (D/100.0)
     #-- U and V are transports in m^2/s
     elif TYPE in ('V','U'):
         unit_conv = 1.0
@@ -231,12 +228,12 @@ def extract_tidal_constants(ilon, ilat, grid_file, model_file, EPSG, TYPE='z',
             if GLOBAL:
                 z = extend_matrix(z)
             #-- interpolate amplitude and phase of the constituent
+            z1 = np.ma.zeros((npts),dtype=z.dtype)
             if (METHOD == 'bilinear'):
                 #-- replace zero values with nan
                 z[z==0] = np.nan
                 #-- use quick bilinear to interpolate values
-                z1 = np.ma.zeros((npts),dtype=z.dtype)
-                z1.data = bilinear_interp(xi,yi,z,x,y,dtype=np.complex128)
+                z1.data[:] = bilinear_interp(xi,yi,z,x,y,dtype=np.complex128)
                 #-- replace nan values with fill_value
                 z1.mask = (np.isnan(z1.data) | (~mz1.astype(np.bool)))
                 z1.data[z1.mask] = z1.fill_value
@@ -246,21 +243,19 @@ def extract_tidal_constants(ilon, ilat, grid_file, model_file, EPSG, TYPE='z',
                     z.real.T,kx=1,ky=1)
                 f2 = scipy.interpolate.RectBivariateSpline(xi,yi,
                     z.imag.T,kx=1,ky=1)
-                z1 = np.ma.zeros((npts),dtype=z.dtype)
                 z1.data.real = f1.ev(x,y)
                 z1.data.imag = f2.ev(x,y)
                 #-- replace zero values with fill_value
                 z1.mask = (~mz1.astype(np.bool))
                 z1.data[z1.mask] = z1.fill_value
             else:
-                #-- replace zero values with nan
-                z[z==0] = np.nan
-                #-- use scipy griddata to interpolate values
+                #-- use scipy regular grid to interpolate values
+                r1 = scipy.interpolate.RegularGridInterpolator((yi,xi),z,
+                    method=METHOD,bounds_error=False,fill_value=z1.fill_value)
                 z1 = np.ma.zeros((npts),dtype=z.dtype)
-                z1.data=scipy.interpolate.griddata(interp_points,
-                    z.flatten(), np.c_[x,y], method=METHOD)
-                #-- replace nan values with fill_value
-                z1.mask = (np.isnan(z1.data) | (~mz1.astype(np.bool)))
+                z1.data[:] = r1.__call__(np.c_[y,x])
+                #-- replace invalid values with fill_value
+                z1.mask = (z1.data == z1.fill_value) | (~mz1.astype(np.bool))
                 z1.data[z1.mask] = z1.fill_value
             #-- amplitude and phase of the constituent
             amplitude.data[:,i] = np.abs(z1.data)
@@ -277,36 +272,35 @@ def extract_tidal_constants(ilon, ilat, grid_file, model_file, EPSG, TYPE='z',
             #-- replace original values with extend matrices
             if GLOBAL:
                 u = extend_matrix(u)
-            #-- interpolate values
+            #-- x coordinates for u transports
+            xu = xi - dx/2.0
+            #-- interpolate amplitude and phase of the constituent
+            u1 = np.ma.zeros((npts),dtype=u.dtype)
             if (METHOD == 'bilinear'):
                 #-- replace zero values with nan
                 u[u==0] = np.nan
                 #-- use quick bilinear to interpolate values
-                u1 = np.ma.zeros((npts),dtype=u.dtype)
-                u1.data = bilinear_interp(xi,yi,u,x,y,dtype=np.complex128)
+                u1.data[:] = bilinear_interp(xu,yi,u,x,y,dtype=np.complex128)
                 #-- replace nan values with fill_value
                 u1.mask = (np.isnan(u1.data) | (~mu1.astype(np.bool)))
                 u1.data[u1.mask] = u1.fill_value
             elif (METHOD == 'spline'):
-                f1 = scipy.interpolate.RectBivariateSpline(xi,yi,
+                f1 = scipy.interpolate.RectBivariateSpline(xu,yi,
                     u.real.T,kx=1,ky=1)
-                f2 = scipy.interpolate.RectBivariateSpline(xi,yi,
+                f2 = scipy.interpolate.RectBivariateSpline(xu,yi,
                     u.imag.T,kx=1,ky=1)
-                u1 = np.ma.zeros((npts),dtype=u.dtype)
                 u1.data.real = f1.ev(x,y)
                 u1.data.imag = f2.ev(x,y)
                 #-- replace zero values with fill_value
                 u1.mask = (~mu1.astype(np.bool))
                 u1.data[u1.mask] = u1.fill_value
             else:
-                #-- replace zero values with nan
-                u[u==0] = np.nan
-                #-- use scipy griddata to interpolate values
-                u1 = np.ma.zeros((npts),dtype=u.dtype)
-                u1.data=scipy.interpolate.griddata(interp_points,
-                    u.flatten(), np.c_[x,y], method=METHOD)
-                #-- replace nan values with fill_value
-                u1.mask = (np.isnan(u1.data) | (~mu1.astype(np.bool)))
+                #-- use scipy regular grid to interpolate values
+                r1 = scipy.interpolate.RegularGridInterpolator((yi,xu),u,
+                    method=METHOD,bounds_error=False,fill_value=u1.fill_value)
+                u1.data[:] = r1.__call__(np.c_[y,x])
+                #-- replace invalid values with fill_value
+                u1.mask = (u1.data == u1.fill_value) | (~mu1.astype(np.bool))
                 u1.data[u1.mask] = u1.fill_value
             #-- convert units
             u1 = u1/unit_conv
@@ -325,36 +319,35 @@ def extract_tidal_constants(ilon, ilat, grid_file, model_file, EPSG, TYPE='z',
             #-- replace original values with extend matrices
             if GLOBAL:
                 v = extend_matrix(v)
-            #-- interpolate values
+            #-- y coordinates for v transports
+            yv = yi - dy/2.0
+            #-- interpolate amplitude and phase of the constituent
+            v1 = np.ma.zeros((npts),dtype=v.dtype)
             if (METHOD == 'bilinear'):
                 #-- replace zero values with nan
                 v[v==0] = np.nan
                 #-- use quick bilinear to interpolate values
-                v1 = np.ma.zeros((npts),dtype=v.dtype)
-                v1.data = bilinear_interp(xi,yi,v,x,y,dtype=np.complex128)
+                v1.data = bilinear_interp(xi,yv,v,x,y,dtype=np.complex128)
                 #-- replace nan values with fill_value
                 v1.mask = (np.isnan(v1.data) | (~mv1.astype(np.bool)))
                 v1.data[v1.mask] = v1.fill_value
             elif (METHOD == 'spline'):
-                f1 = scipy.interpolate.RectBivariateSpline(xi,yi,
+                f1 = scipy.interpolate.RectBivariateSpline(xi,yv,
                     v.real.T,kx=1,ky=1)
-                f2 = scipy.interpolate.RectBivariateSpline(xi,yi,
+                f2 = scipy.interpolate.RectBivariateSpline(xi,yv,
                     v.imag.T,kx=1,ky=1)
-                v1 = np.ma.zeros((npts),dtype=v.dtype)
                 v1.data.real = f1.ev(x,y)
                 v1.data.imag = f2.ev(x,y)
                 #-- replace zero values with fill_value
                 v1.mask = (~mv1.astype(np.bool))
                 v1.data[v1.mask] = v1.fill_value
             else:
-                #-- replace zero values with nan
-                v[u==0] = np.nan
-                #-- use scipy griddata to interpolate values
-                v1 = np.ma.zeros((npts),dtype=v.dtype)
-                v1.data=scipy.interpolate.griddata(interp_points,
-                    v.flatten(), np.c_[x,y], method=METHOD)
-                #-- replace nan values with fill_value
-                v1.mask = (np.isnan(v1.data) | (~mv1.astype(np.bool)))
+                #-- use scipy regular grid to interpolate values
+                r1 = scipy.interpolate.RegularGridInterpolator((yv,xi),v,
+                    method=METHOD,bounds_error=False,fill_value=v1.fill_value)
+                v1.data[:] = r1.__call__(np.c_[y,x])
+                #-- replace invalid values with fill_value
+                v1.mask = (v1.data == v1.fill_value) | (~mv1.astype(np.bool))
                 v1.data[v1.mask] = v1.fill_value
             #-- convert units
             v1 = v1/unit_conv
@@ -363,6 +356,9 @@ def extract_tidal_constants(ilon, ilat, grid_file, model_file, EPSG, TYPE='z',
             amplitude.mask[:,i] = np.copy(v1.mask)
             ph.data[:,i] = np.arctan2(-np.imag(v1),np.real(v1))
             ph.mask[:,i] = np.copy(v1.mask)
+        #-- update mask to invalidate points outside model domain
+        ph.mask[:,i] |= invalid
+        amplitude.mask[:,i] |= invalid
 
     #-- convert phase to degrees
     phase = ph*180.0/np.pi
@@ -550,8 +546,10 @@ def read_atlas_grid(input_file):
         iz = np.fromfile(fid, dtype=np.dtype('>i4'), count=nd)
         jz = np.fromfile(fid, dtype=np.dtype('>i4'), count=nd)
         fid.seek(8,1)
-        depth = np.full((ny1,nx1),np.nan)
-        depth[jz-1,iz-1] = np.fromfile(fid, dtype=np.dtype('>f4'), count=nd)
+        depth = np.ma.zeros((ny1,nx1))
+        depth.mask = np.ones((ny1,nx1),dtype=np.bool)
+        depth.data[jz-1,iz-1] = np.fromfile(fid,dtype=np.dtype('>f4'),count=nd)
+        depth.mask[jz-1,iz-1] = False
         fid.seek(4,1)
         #-- save to dictionary
         local[name] = dict(lon=ln1,lat=lt1,depth=depth)
@@ -692,10 +690,11 @@ def read_atlas_elevation(input_file,ic,constituent):
             fid.seek(nskip,1)
             #-- real and imaginary components of elevation
             h1 = np.ma.zeros((ny1,nx1),fill_value=np.nan,dtype=np.complex64)
-            h1.mask = np.zeros((ny1,nx1),dtype=np.bool)
+            h1.mask = np.ones((ny1,nx1),dtype=np.bool)
             temp = np.fromfile(fid, dtype=np.dtype('>f4'), count=2*nz)
             h1.data.real[jz-1,iz-1] = temp[0:2*nz-1:2]
             h1.data.imag[jz-1,iz-1] = temp[1:2*nz:2]
+            h1.mask[jz-1,iz-1] = False
             #-- save constituent to dictionary
             local[name] = dict(lon=ln1,lat=lt1,z=h1)
             #-- skip records after constituent
@@ -834,17 +833,19 @@ def read_atlas_transport(input_file,ic,constituent):
             fid.seek(nskip,1)
             #-- real and imaginary components of u transport
             u1 = np.ma.zeros((ny1,nx1),fill_value=np.nan,dtype=np.complex64)
-            u1.mask = np.zeros((ny1,nx1),dtype=np.bool)
+            u1.mask = np.ones((ny1,nx1),dtype=np.bool)
             tmpu = np.fromfile(fid, dtype=np.dtype('>f4'), count=2*nu)
             u1.data.real[ju-1,iu-1] = tmpu[0:2*nu-1:2]
             u1.data.imag[ju-1,iu-1] = tmpu[1:2*nu:2]
+            u1.mask[ju-1,iu-1] = False
             fid.seek(8,1)
             #-- real and imaginary components of v transport
             v1 = np.ma.zeros((ny1,nx1),fill_value=np.nan,dtype=np.complex64)
-            v1.mask = np.zeros((ny1,nx1),dtype=np.bool)
+            v1.mask = np.ones((ny1,nx1),dtype=np.bool)
             tmpv = np.fromfile(fid, dtype=np.dtype('>f4'), count=2*nv)
             v1.data.real[jv-1,iv-1] = tmpv[0:2*nv-1:2]
             v1.data.imag[jv-1,iv-1] = tmpv[1:2*nv:2]
+            v1.mask[jv-1,iv-1] = False
             #-- save constituent to dictionary
             local[name] = dict(lon=ln1,lat=lt1,u=u1,v=v1)
             #-- skip records after constituent
@@ -887,18 +888,20 @@ def create_atlas_mask(xi,yi,mz,local,VARIABLE=None):
     x30 = np.arange(d30/2.0, 360.0+d30/2.0, d30)
     y30 = np.arange(-90.0+d30/2.0, 90.0+d30/2.0, d30)
     #-- interpolate global mask to create initial 2 arc-minute mask
-    m30 = np.zeros((len(y30),len(x30)),dtype=mz.dtype)
-    f = scipy.interpolate.RectBivariateSpline(xi,yi,mz.T,kx=1,ky=1)
-    m30[:,:] = f(x30,y30).T
+    xcoords=np.clip((len(xi)-1)*(x30-xi[0])/(xi[-1]-xi[0]),0,len(xi)-1)
+    ycoords=np.clip((len(yi)-1)*(y30-yi[0])/(yi[-1]-yi[0]),0,len(yi)-1)
+    gridy,gridx=np.meshgrid(np.around(ycoords),np.around(xcoords),indexing='ij')
+    #-- interpolate with nearest-neighbors
+    m30 = np.ma.zeros((len(y30),len(x30)),dtype=np.int8,fill_value=0)
+    m30.data[:,:] = mz[gridy.astype(np.int32),gridx.astype(np.int32)]
     #-- iterate over localized solutions to fill in high-resolution coastlines
     for key,val in local.items():
-        #-- local model output
-        zlocal = val[VARIABLE][:]
-        validy,validx = np.nonzero(np.isfinite(zlocal.real))
         #-- create latitude and longitude for local model
         ilon = np.arange(val['lon'][0]+d30/2.0,val['lon'][1]+d30/2.0,d30)
         ilat = np.arange(val['lat'][0]+d30/2.0,val['lat'][1]+d30/2.0,d30)
         X,Y = np.meshgrid(ilon,ilat)
+        #-- local model output
+        validy,validx = np.nonzero(~val[VARIABLE].mask)
         for indy,indx in zip(validy,validx):
             #-- check if model is -180:180
             lon30 = (X[indy,indx]+360.) if (X[indy,indx]<=0.0) else X[indy,indx]
@@ -907,6 +910,7 @@ def create_atlas_mask(xi,yi,mz,local,VARIABLE=None):
             #-- fill global mask with regional solution
             m30[jj,ii] = 1
     #-- return the 2 arc-minute mask
+    m30.mask = (m30.data == m30.fill_value)
     return m30
 
 #-- PURPOSE: combines global and local atlas solutions
@@ -957,7 +961,7 @@ def combine_atlas_model(xi,yi,zi,pmask,local,VARIABLE=None):
     for key,val in local.items():
         #-- local model output
         zlocal = val[VARIABLE][:]
-        validy,validx = np.nonzero(np.isfinite(zlocal.data.real))
+        validy,validx = np.nonzero(~zlocal.mask)
         #-- create latitude and longitude for local model
         ilon = np.arange(val['lon'][0]+d30/2.0,val['lon'][1]+d30/2.0,d30)
         ilat = np.arange(val['lat'][0]+d30/2.0,val['lat'][1]+d30/2.0,d30)
