@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 u"""
 compute_LPT_icebridge_data.py
-Written by Tyler Sutterley (08/2020)
+Written by Tyler Sutterley (11/2020)
 Calculates load pole tide displacements for correcting Operation IceBridge
     elevation data following IERS Convention (2010) guidelines
     http://maia.usno.navy.mil/conventions/2010officialinfo.php
@@ -11,8 +11,7 @@ INPUTS:
     ATM1B, ATM icessn or LVIS file from NSIDC
 
 COMMAND LINE OPTIONS:
-    -D X, --directory=X: Working data directory
-    -M X, --mode=X: Permission mode of directories and files created
+    -M X, --mode X: Permission mode of directories and files created
     -V, --verbose: Output information about each created file
 
 PYTHON DEPENDENCIES:
@@ -34,6 +33,9 @@ PROGRAM DEPENDENCIES:
     read_ATM1b_QFIT_binary.py: read ATM1b QFIT binary files (NSIDC version 1)
 
 UPDATE HISTORY:
+    Updated 11/2020: use internal mean pole and finals EOP files
+    Updated 10/2020: using argparse to set command line parameters
+    Updated 09/2020: output modified julian days as time variable
     Updated 08/2020: using builtin time operations.  python3 regular expressions
     Updated 03/2020: use read_ATM1b_QFIT_binary from repository
     Updated 02/2019: using range for python3 compatibility
@@ -47,10 +49,10 @@ import os
 import re
 import time
 import h5py
-import getopt
+import argparse
 import numpy as np
-import scipy.interpolate
 import pyTMD.time
+import scipy.interpolate
 from pyTMD.convert_julian import convert_julian
 from pyTMD.convert_calendar_decimal import convert_calendar_decimal
 from pyTMD.iers_mean_pole import iers_mean_pole
@@ -352,7 +354,7 @@ def read_LVIS_HDF5_file(input_file, input_subsetter):
 
 #-- PURPOSE: read Operation IceBridge data from NSIDC
 #-- compute load pole tide radial displacements at data points and times
-def compute_LPT_icebridge_data(tide_dir, arg, VERBOSE=False, MODE=0o775):
+def compute_LPT_icebridge_data(arg, VERBOSE=False, MODE=0o775):
 
     #-- extract file name and subsetter indices lists
     match_object = re.match(r'(.*?)(\[(.*?)\])?$',arg)
@@ -403,10 +405,11 @@ def compute_LPT_icebridge_data(tide_dir, arg, VERBOSE=False, MODE=0o775):
         'chapter7/opoleloadcoefcmcor.txt.gz')
     attrib['tide_pole']['units'] = 'meters'
     #-- Modified Julian Days
-    attrib['MJD'] = {}
-    attrib['MJD']['long_name'] = 'Time'
-    attrib['MJD']['description'] = 'Modified Julian Days'
-    attrib['MJD']['units'] = 'Days'
+    attrib['time'] = {}
+    attrib['time']['long_name'] = 'Time'
+    attrib['time']['units'] = 'days since 1858-11-17T00:00:00'
+    attrib['time']['description'] = 'Modified Julian Days'
+    attrib['time']['calendar'] = 'standard'
 
     #-- extract information from first input file
     #-- acquisition year, month and day
@@ -450,14 +453,14 @@ def compute_LPT_icebridge_data(tide_dir, arg, VERBOSE=False, MODE=0o775):
     #-- convert calendar dates into year decimal
     tdec = convert_calendar_decimal(YY,MM,DAY=DD,HOUR=HH,MINUTE=MN,SECOND=SS)
     #-- elevation
-    h1 = dinput['data'][:]
+    h1 = np.copy(dinput['data'][:])
 
     #-- degrees to radians
     dtr = np.pi/180.0
     atr = np.pi/648000.0
     #-- earth and physical parameters (IERS and WGS84)
     G = 6.67428e-11#-- universal constant of gravitation [m^3/(kg*s^2)]
-    GM = 3.98004418e14#-- geocentric gravitational constant [m^3/s^2]
+    GM = 3.986004418e14#-- geocentric gravitational constant [m^3/s^2]
     ge = 9.7803278#-- mean equatorial gravity [m/s^2]
     a_axis = 6378136.6#-- semimajor axis of the WGS84 ellipsoid [m]
     flat = 1.0/298.257223563#-- flattening of the WGS84 ellipsoid
@@ -499,13 +502,13 @@ def compute_LPT_icebridge_data(tide_dir, arg, VERBOSE=False, MODE=0o775):
     gamma_0 = gamma_a*(1.0 + f_2*np.cos(theta)**2.0 +
         f_4*np.sin(np.pi*latitude_geocentric/180.0)**4.0)
     #-- Normal gravity at height h. p. 82, Eqn.(2-215)
-    gamma_h = gamma_0*(1.0-(2.0/a_axis)*(1.0+flat+m-2.0*flat*np.cos(theta)**2.0)
-        *h1 + (3.0/a_axis**2.0)*h1**2.0)
+    gamma_h = gamma_0*(1.0 -
+        (2.0/a_axis)*(1.0+flat+m-2.0*flat*np.cos(theta)**2.0)*h1 + \
+        (3.0/a_axis**2.0)*h1**2.0)
 
     #-- pole tide files (mean and daily)
-    # mean_pole_file = os.path.join(tide_dir,'mean-pole.tab')
-    mean_pole_file = os.path.join(tide_dir,'mean_pole_2017-10-23.tab')
-    pole_tide_file = os.path.join(tide_dir,'finals_all_2017-09-01.tab')
+    mean_pole_file = pyTMD.utilities.get_data_path(['data','mean-pole.tab'])
+    pole_tide_file = pyTMD.utilities.get_data_path(['data','finals.all'])
     #-- read IERS daily polar motion values
     EOP = read_iers_EOP(pole_tide_file)
     #-- create cubic spline interpolations of daily polar motion values
@@ -542,7 +545,11 @@ def compute_LPT_icebridge_data(tide_dir, arg, VERBOSE=False, MODE=0o775):
     my = -(py - mpy)
     #-- calculate radial displacement at time
     dfactor = -hb2*atr*(omega**2*rr**2)/(2.0*gamma_h)
-    Sr = dfactor*np.sin(2.0*theta)*(mx*np.cos(phi) + my*np.sin(phi))
+    Srad = np.ma.zeros((file_lines),fill_value=fill_value)
+    Srad.data[:] = dfactor*np.sin(2.0*theta)*(mx*np.cos(phi) + my*np.sin(phi))
+    #-- replace fill values
+    Srad.mask = np.isnan(Srad.data)
+    Srad.data[Srad.mask] = Srad.fill_value
 
     #-- add latitude and longitude to output file
     for key in ['lat','lon']:
@@ -556,8 +563,8 @@ def compute_LPT_icebridge_data(tide_dir, arg, VERBOSE=False, MODE=0o775):
         h5.dims[0].label = 'RECORD_SIZE'
 
     #-- output tides to HDF5 dataset
-    h5 = fid.create_dataset('tide_pole', (file_lines,), data=Sr,
-        dtype=Sr.dtype, compression='gzip')
+    h5 = fid.create_dataset('tide_pole', (file_lines,), data=Srad,
+        dtype=Srad.dtype, compression='gzip')
     #-- add HDF5 variable attributes
     h5.attrs['_FillValue'] = fill_value
     for att_name,att_val in attrib['tide_pole'].items():
@@ -566,10 +573,10 @@ def compute_LPT_icebridge_data(tide_dir, arg, VERBOSE=False, MODE=0o775):
     h5.dims[0].label = 'RECORD_SIZE'
 
     #-- output days to HDF5 dataset
-    h5 = fid.create_dataset('MJD', (file_lines,), data=t, dtype=t.dtype,
+    h5 = fid.create_dataset('time', (file_lines,), data=t, dtype=t.dtype,
         compression='gzip')
     #-- add HDF5 variable attributes
-    for att_name,att_val in attrib['MJD'].items():
+    for att_name,att_val in attrib['time'].items():
         h5.attrs[att_name] = att_val
     #-- attach dimensions
     h5.dims[0].label = 'RECORD_SIZE'
@@ -615,43 +622,32 @@ def compute_LPT_icebridge_data(tide_dir, arg, VERBOSE=False, MODE=0o775):
     #-- change the permissions level to MODE
     os.chmod(os.path.join(DIRECTORY,FILENAME), MODE)
 
-#-- PURPOSE: help module to describe the optional input parameters
-def usage():
-    print('\nHelp: {}'.format(os.path.basename(sys.argv[0])))
-    print(' -D X, --directory=X\tWorking data directory')
-    print(' -M X, --mode=X\t\tPermission mode of directories and files created')
-    print(' -V, --verbose\t\tOutput information about each created file\n')
-
 #-- Main program that calls compute_LPT_icebridge_data()
 def main():
     #-- Read the system arguments listed after the program
-    long_options = ['help','directory=','verbose','mode=']
-    optlist,arglist = getopt.getopt(sys.argv[1:], 'hD:VM:', long_options)
-
-    #-- directory with tide data
-    tide_dir = os.getcwd()
+    parser = argparse.ArgumentParser(
+        description="""Calculates load pole tide displacements for
+            correcting Operation IceBridge elevation data following IERS
+            Convention (2010) guidelines
+            """
+    )
+    #-- command line options
+    parser.add_argument('infile',
+        type=lambda p: os.path.abspath(os.path.expanduser(p)), nargs='+',
+        help='Input Operation IceBridge file')
     #-- verbosity settings
-    VERBOSE = False
+    parser.add_argument('--verbose','-V',
+        default=False, action='store_true',
+        help='Output information about each created file')
     #-- permissions mode of the local files (number in octal)
-    MODE = 0o775
-    for opt, arg in optlist:
-        if opt in ('-h','--help'):
-            usage()
-            sys.exit()
-        elif opt in ("-D","--directory"):
-            tide_dir = os.path.expanduser(arg)
-        elif opt in ("-V","--verbose"):
-            VERBOSE = True
-        elif opt in ("-M","--mode"):
-            MODE = int(arg, 8)
-
-    #-- enter input file from NSIDC as system argument
-    if not arglist:
-        raise Exception('No System Arguments Listed')
+    parser.add_argument('--mode','-M',
+        type=lambda x: int(x,base=8), default=0o775,
+        help='Permission mode of output file')
+    args = parser.parse_args()
 
     #-- run for each input file
-    for arg in arglist:
-        compute_LPT_icebridge_data(tide_dir, arg, VERBOSE=VERBOSE, MODE=MODE)
+    for arg in args.infile:
+        compute_LPT_icebridge_data(arg, VERBOSE=args.verbose, MODE=args.mode)
 
 #-- run main program
 if __name__ == '__main__':
