@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 u"""
 compute_OPT_icebridge_data.py
-Written by Tyler Sutterley (08/2020)
+Written by Tyler Sutterley (10/2020)
 Calculates radial ocean pole tide displacements for correcting Operation
     IceBridge elevation data following IERS Convention (2010) guidelines
     http://maia.usno.navy.mil/conventions/2010officialinfo.php
@@ -11,9 +11,11 @@ INPUTS:
     ATM1B, ATM icessn or LVIS file from NSIDC
 
 COMMAND LINE OPTIONS:
-    -D X, --directory=X: Working data directory
-    -I X, --interpolate=X: Interpolation method (default spline)
-    -M X, --mode=X: Permission mode of directories and files created
+    -I X, --interpolate X: Interpolation method
+        spline
+        linear
+        nearest
+    -M X, --mode X: Permission mode of directories and files created
     -V, --verbose: Output information about each created file
 
 PYTHON DEPENDENCIES:
@@ -36,7 +38,11 @@ PROGRAM DEPENDENCIES:
     read_ATM1b_QFIT_binary.py: read ATM1b QFIT binary files (NSIDC version 1)
 
 UPDATE HISTORY:
+    Updated 11/2020: use internal mean pole and finals EOP files
+    Updated 10/2020: using argparse to set command line parameters
+    Updated 09/2020: output modified julian days as time variable
     Updated 08/2020: using builtin time operations.  python3 regular expressions
+        replaced griddata interpolation with scipy regular grid interpolators
     Updated 03/2020: use read_ATM1b_QFIT_binary from repository
     Updated 05/2019: added option interpolate to choose the interpolation method
     Updated 02/2019: using range for python3 compatibility
@@ -51,10 +57,11 @@ import re
 import time
 import gzip
 import h5py
-import getopt
+import argparse
 import numpy as np
 import scipy.interpolate
 import pyTMD.time
+from pyTMD.utilities import get_data_path
 from pyTMD.convert_julian import convert_julian
 from pyTMD.convert_calendar_decimal import convert_calendar_decimal
 from pyTMD.iers_mean_pole import iers_mean_pole
@@ -357,7 +364,7 @@ def read_LVIS_HDF5_file(input_file, input_subsetter):
 
 #-- PURPOSE: read Operation IceBridge data from NSIDC
 #-- compute ocean pole tide radial displacements at data points and times
-def compute_OPT_icebridge_data(tide_dir,arg,METHOD=None,VERBOSE=False,MODE=0o775):
+def compute_OPT_icebridge_data(arg,METHOD=None,VERBOSE=False,MODE=0o775):
 
     #-- extract file name and subsetter indices lists
     match_object = re.match(r'(.*?)(\[(.*?)\])?$',arg)
@@ -384,7 +391,10 @@ def compute_OPT_icebridge_data(tide_dir,arg,METHOD=None,VERBOSE=False,MODE=0o775
         if re.match(val, os.path.basename(input_file)):
             OIB = key
 
-    #-- HDF5 file attributes
+    #-- invalid value
+    fill_value = -9999.0
+    #-- output netCDF4 and HDF5 file attributes
+    #-- will be added to YAML header in csv files
     attrib = {}
     #-- latitude
     attrib['lat'] = {}
@@ -408,10 +418,11 @@ def compute_OPT_icebridge_data(tide_dir,arg,METHOD=None,VERBOSE=False,MODE=0o775
         'chapter7/opoleloadcoefcmcor.txt.gz')
     attrib['tide_oc_pole']['units'] = 'meters'
     #-- Modified Julian Days
-    attrib['MJD'] = {}
-    attrib['MJD']['long_name'] = 'Time'
-    attrib['MJD']['description'] = 'Modified Julian Days'
-    attrib['MJD']['units'] = 'Days'
+    attrib['time'] = {}
+    attrib['time']['long_name'] = 'Time'
+    attrib['time']['units'] = 'days since 1858-11-17T00:00:00'
+    attrib['time']['description'] = 'Modified Julian Days'
+    attrib['time']['calendar'] = 'standard'
 
     #-- extract information from first input file
     #-- acquisition year, month and day
@@ -462,12 +473,13 @@ def compute_OPT_icebridge_data(tide_dir,arg,METHOD=None,VERBOSE=False,MODE=0o775
     atr = np.pi/648000.0
     #-- earth and physical parameters (IERS)
     G = 6.67428e-11#-- universal constant of gravitation [m^3/(kg*s^2)]
-    GM = 3.98004418e14#-- geocentric gravitational constant [m^3/s^2]
+    GM = 3.986004418e14#-- geocentric gravitational constant [m^3/s^2]
     ge = 9.7803278#-- mean equatorial gravity [m/s^2]
     a_axis = 6378136.6#-- equatorial radius of the Earth [m]
     flat = 1.0/298.257223563#-- flattening of the ellipsoid
     omega = 7.292115e-5#-- mean rotation rate of the Earth [radians/s]
     rho_w = 1025.0#-- density of sea water [kg/m^3]
+    ge = 9.7803278#-- mean equatorial gravitational acceleration [m/s^2]
     #-- Linear eccentricity and first numerical eccentricity
     lin_ecc = np.sqrt((2.0*flat - flat**2)*a_axis**2)
     ecc1 = lin_ecc/a_axis
@@ -489,16 +501,16 @@ def compute_OPT_icebridge_data(tide_dir,arg,METHOD=None,VERBOSE=False,MODE=0o775
 
     #-- pole tide displacement scale factor
     Hp = np.sqrt(8.0*np.pi/15.0)*(omega**2*a_axis**4)/GM
-    K = 4.0*np.pi*G*rho_w*Hp*a_axis**3/(3.0*GM)
+    K = 4.0*np.pi*G*rho_w*Hp*a_axis/(3.0*ge)
+    K1 = 4.0*np.pi*G*rho_w*Hp*a_axis**3/(3.0*GM)
 
     #-- read ocean pole tide map from Desai (2002)
-    ocean_pole_tide_file = os.path.join(tide_dir,'opoleloadcoefcmcor.txt.gz')
-    iur,ilon,ilat = read_ocean_pole_tide(ocean_pole_tide_file)
+    ocean_pole_tide_file = get_data_path(['data','opoleloadcoefcmcor.txt.gz'])
+    iur,iun,iue,ilon,ilat = read_ocean_pole_tide(ocean_pole_tide_file)
 
     #-- pole tide files (mean and daily)
-    # mean_pole_file = os.path.join(tide_dir,'mean-pole.tab')
-    mean_pole_file = os.path.join(tide_dir,'mean_pole_2017-10-23.tab')
-    pole_tide_file = os.path.join(tide_dir,'finals_all_2017-09-01.tab')
+    mean_pole_file = pyTMD.utilities.get_data_path(['data','mean-pole.tab'])
+    pole_tide_file = pyTMD.utilities.get_data_path(['data','finals.all'])
 
     #-- read IERS daily polar motion values
     EOP = read_iers_EOP(pole_tide_file)
@@ -537,12 +549,10 @@ def compute_OPT_icebridge_data(tide_dir,arg,METHOD=None,VERBOSE=False,MODE=0o775
         UR.real = f1.ev(lon,latitude_geocentric)
         UR.imag = f2.ev(lon,latitude_geocentric)
     else:
-        #-- create mesh grids of latitude and longitude
-        gridlon,gridlat = np.meshgrid(ilon, ilat, indexing='ij')
-        interp_points = zip(gridlon.flatten(),gridlat.flatten())
-        #-- use scipy griddata to interpolate to output points
-        UR = scipy.interpolate.griddata(interp_points, iur.flatten(),
-            zip(lon,latitude_geocentric), method=METHOD)
+        #-- use scipy regular grid to interpolate values for a given method
+        r1 = scipy.interpolate.RegularGridInterpolator((ilon,ilat[::-1]),
+            iur[:,::-1], method=METHOD)
+        UR = r1.__call__(np.c_[lon,latitude_geocentric])
 
     #-- calculate angular coordinates of mean pole at time tdec
     mpx,mpy,fl = iers_mean_pole(mean_pole_file,tdec,'2015')
@@ -553,12 +563,12 @@ def compute_OPT_icebridge_data(tide_dir,arg,METHOD=None,VERBOSE=False,MODE=0o775
     mx = px - mpx
     my = -(py - mpy)
     #-- calculate radial displacement at time
-    U = K*atr*np.real((mx*gamma.real + my*gamma.imag)*UR.real +
+    Urad = np.ma.zeros((file_lines),fill_value=fill_value)
+    Urad.data[:] = K*atr*np.real((mx*gamma.real + my*gamma.imag)*UR.real +
         (my*gamma.real - mx*gamma.imag)*UR.imag)
-
-    #-- replace nan values with fill value
-    inan, = np.nonzero(np.isnan(U))
-    U[inan] = fill_value
+    #-- replace fill values
+    Urad.mask = np.isnan(Urad.data)
+    Urad.data[Urad.mask] = Urad.fill_value
 
     #-- add latitude and longitude to output file
     for key in ['lat','lon']:
@@ -572,8 +582,8 @@ def compute_OPT_icebridge_data(tide_dir,arg,METHOD=None,VERBOSE=False,MODE=0o775
         h5.dims[0].label = 'RECORD_SIZE'
 
     #-- output tides to HDF5 dataset
-    h5 = fid.create_dataset('tide_oc_pole', (file_lines,), data=U,
-        dtype=U.dtype, fillvalue=fill_value, compression='gzip')
+    h5 = fid.create_dataset('tide_oc_pole', (file_lines,), data=Urad,
+        dtype=Urad.dtype, fillvalue=fill_value, compression='gzip')
     #-- add HDF5 variable attributes
     h5.attrs['_FillValue'] = fill_value
     for att_name,att_val in attrib['tide_oc_pole'].items():
@@ -582,10 +592,10 @@ def compute_OPT_icebridge_data(tide_dir,arg,METHOD=None,VERBOSE=False,MODE=0o775
     h5.dims[0].label = 'RECORD_SIZE'
 
     #-- output days to HDF5 dataset
-    h5 = fid.create_dataset('MJD', (file_lines,), data=t, dtype=t.dtype,
+    h5 = fid.create_dataset('time', (file_lines,), data=t, dtype=t.dtype,
         compression='gzip')
     #-- add HDF5 variable attributes
-    for att_name,att_val in attrib['MJD'].items():
+    for att_name,att_val in attrib['time'].items():
         h5.attrs[att_name] = att_val
     #-- attach dimensions
     h5.dims[0].label = 'RECORD_SIZE'
@@ -631,49 +641,38 @@ def compute_OPT_icebridge_data(tide_dir,arg,METHOD=None,VERBOSE=False,MODE=0o775
     #-- change the permissions level to MODE
     os.chmod(os.path.join(DIRECTORY,FILENAME), MODE)
 
-#-- PURPOSE: help module to describe the optional input parameters
-def usage():
-    print('\nHelp: {}'.format(os.path.basename(sys.argv[0])))
-    print(' -D X, --directory=X\tWorking data directory')
-    print(' -I X, --interpolate\tInterpolation method (default spline)')
-    print(' -M X, --mode=X\t\tPermission mode of directories and files created')
-    print(' -V, --verbose\t\tOutput information about each created file\n')
-
 #-- Main program that calls compute_OPT_icebridge_data()
 def main():
     #-- Read the system arguments listed after the program
-    long_options = ['help','directory=','interpolate=','verbose','mode=']
-    optlist,arglist = getopt.getopt(sys.argv[1:], 'hD:I:VM:', long_options)
-
-    #-- directory with tide data
-    tide_dir = os.getcwd()
+    parser = argparse.ArgumentParser(
+        description="""Calculates radial ocean pole tide displacements for
+            correcting Operation IceBridge elevation data following IERS
+            Convention (2010) guidelines
+            """
+    )
+    #-- command line options
+    parser.add_argument('infile',
+        type=lambda p: os.path.abspath(os.path.expanduser(p)), nargs='+',
+        help='Input Operation IceBridge file')
     #-- interpolation method
-    METHOD = 'spline'
+    parser.add_argument('--interpolate','-I',
+        metavar='METHOD', type=str, default='spline',
+        choices=('spline','linear','nearest'),
+        help='Spatial interpolation method')
     #-- verbosity settings
-    VERBOSE = False
+    parser.add_argument('--verbose','-V',
+        default=False, action='store_true',
+        help='Output information about each created file')
     #-- permissions mode of the local files (number in octal)
-    MODE = 0o775
-    for opt, arg in optlist:
-        if opt in ('-h','--help'):
-            usage()
-            sys.exit()
-        elif opt in ("-D","--directory"):
-            tide_dir = os.path.expanduser(arg)
-        elif opt in ("-I","--interpolate"):
-            METHOD = arg.lower()
-        elif opt in ("-V","--verbose"):
-            VERBOSE = True
-        elif opt in ("-M","--mode"):
-            MODE = int(arg, 8)
-
-    #-- enter input file from NSIDC as system argument
-    if not arglist:
-        raise Exception('No System Arguments Listed')
+    parser.add_argument('--mode','-M',
+        type=lambda x: int(x,base=8), default=0o775,
+        help='Permission mode of output file')
+    args = parser.parse_args()
 
     #-- run for each input file
-    for arg in arglist:
-        compute_OPT_icebridge_data(tide_dir, os.path.expanduser(arg),
-            METHOD=METHOD, VERBOSE=VERBOSE, MODE=MODE)
+    for arg in args.infile:
+        compute_OPT_icebridge_data(arg, METHOD=args.interpolate,
+            VERBOSE=args.verbose, MODE=args.mode)
 
 #-- run main program
 if __name__ == '__main__':
