@@ -1,26 +1,28 @@
 #!/usr/bin/env python
 u"""
 compute_tides_icebridge_data.py
-Written by Tyler Sutterley (08/2020)
+Written by Tyler Sutterley (11/2020)
 Calculates tidal elevations for correcting Operation IceBridge elevation data
 
 Uses OTIS format tidal solutions provided by Ohio State University and ESR
     http://volkov.oce.orst.edu/tides/region.html
     https://www.esr.org/research/polar-tide-models/list-of-polar-tide-models/
     ftp://ftp.esr.org/pub/datasets/tmd/
-or Global Tide Model (GOT) solutions provided by Richard Ray at GSFC
+Global Tide Model (GOT) solutions provided by Richard Ray at GSFC
+or Finite Element Solution (FES) models provided by AVISO
 
 INPUTS:
     ATM1B, ATM icessn or LVIS file from NSIDC
 
 COMMAND LINE OPTIONS:
-    -D X, --directory=X: Working data directory
-    -T X, --tide=X: Tide model to use in correction
+    -D X, --directory X: Working data directory
+    -T X, --tide X: Tide model to use in correction
         CATS0201
         CATS2008
         CATS2008_load
-        TPXO9-atlas-v2
         TPXO9-atlas
+        TPXO9-atlas-v2
+        TPXO9-atlas-v3
         TPXO9.1
         TPXO8-atlas
         TPXO7.2
@@ -36,7 +38,12 @@ COMMAND LINE OPTIONS:
         GOT4.10_load
         FES2014
         FES2014_load
-    -M X, --mode=X: Permission mode of directories and files created
+    -I X, --interpolate X: Interpolation method
+        spline
+        linear
+        nearest
+        bilinear
+    -M X, --mode X: Permission mode of directories and files created
     -V, --verbose: Output information about each created file
 
 PYTHON DEPENDENCIES:
@@ -70,9 +77,12 @@ PROGRAM DEPENDENCIES:
     read_ATM1b_QFIT_binary.py: read ATM1b QFIT binary files (NSIDC version 1)
 
 UPDATE HISTORY:
+    Updated 11/2020: added model constituents from TPXO9-atlas-v3
+    Updated 10/2020: using argparse to set command line parameters
+    Updated 09/2020: output ocean and load tide as tide_ocean and tide_load
     Updated 08/2020: using builtin time operations.  python3 regular expressions
     Updated 07/2020: added FES2014 and FES2014_load.  use merged delta times
-    Updated 06/2020: added version 2 of TPX09-atlas (TPX09-atlas-v2)
+    Updated 06/2020: added version 2 of TPXO9-atlas (TPXO9-atlas-v2)
     Updated 03/2020: use read_ATM1b_QFIT_binary from repository
     Updated 02/2020: changed CATS2008 grid to match version on U.S. Antarctic
         Program Data Center http://www.usap-dc.org/view/dataset/601235
@@ -91,10 +101,10 @@ import os
 import re
 import time
 import h5py
-import getopt
+import argparse
 import numpy as np
 import pyTMD.time
-import pyTMD.utilities
+from pyTMD.utilities import get_data_path
 from pyTMD.convert_julian import convert_julian
 from read_ATM1b_QFIT_binary.read_ATM1b_QFIT_binary import read_ATM1b_QFIT_binary
 from pyTMD.calc_delta_time import calc_delta_time
@@ -400,8 +410,8 @@ def read_LVIS_HDF5_file(input_file, input_subsetter):
 
 #-- PURPOSE: read Operation IceBridge data from NSIDC
 #-- compute tides at points and times using tidal model driver algorithms
-def compute_tides_icebridge_data(tide_dir, arg, MODEL, METHOD=None,
-    VERBOSE=False, MODE=0o775):
+def compute_tides_icebridge_data(tide_dir, arg, TIDE_MODEL,
+    METHOD='spline', VERBOSE=False, MODE=0o775):
 
     #-- extract file name and subsetter indices lists
     match_object = re.match(r'(.*?)(\[(.*?)\])?$',arg)
@@ -428,36 +438,37 @@ def compute_tides_icebridge_data(tide_dir, arg, MODEL, METHOD=None,
         if re.match(val, os.path.basename(input_file)):
             OIB = key
 
-    #-- HDF5 file attributes
-    attrib = dict(lon={},lat={},tide={},day={})
     #-- select between tide models
-    if (MODEL == 'CATS0201'):
+    if (TIDE_MODEL == 'CATS0201'):
         grid_file = os.path.join(tide_dir,'cats0201_tmd','grid_CATS')
         model_file = os.path.join(tide_dir,'cats0201_tmd','h0_CATS02_01')
         reference = 'https://mail.esr.org/polar_tide_models/Model_CATS0201.html'
-        attrib['tide']['long_name'] = 'Ocean_Tide'
+        output_variable = 'tide_ocean'
+        variable_long_name = 'Ocean_Tide'
         model_format = 'OTIS'
         EPSG = '4326'
         TYPE = 'z'
-    elif (MODEL == 'CATS2008'):
+    elif (TIDE_MODEL == 'CATS2008'):
         grid_file = os.path.join(tide_dir,'CATS2008','grid_CATS2008')
         model_file = os.path.join(tide_dir,'CATS2008','hf.CATS2008.out')
         reference = ('https://www.esr.org/research/polar-tide-models/'
             'list-of-polar-tide-models/cats2008/')
-        attrib['tide']['long_name'] = 'Ocean_Tide'
+        output_variable = 'tide_ocean'
+        variable_long_name = 'Ocean_Tide'
         model_format = 'OTIS'
         EPSG = 'CATS2008'
         TYPE = 'z'
-    elif (MODEL == 'CATS2008_load'):
+    elif (TIDE_MODEL == 'CATS2008_load'):
         grid_file = os.path.join(tide_dir,'CATS2008a_SPOTL_Load','grid_CATS2008a_opt')
         model_file = os.path.join(tide_dir,'CATS2008a_SPOTL_Load','h_CATS2008a_SPOTL_load')
         reference = ('https://www.esr.org/research/polar-tide-models/'
             'list-of-polar-tide-models/cats2008/')
-        attrib['tide']['long_name'] = 'Load_Tide'
+        output_variable = 'tide_load'
+        variable_long_name = 'Load_Tide'
         model_format = 'OTIS'
         EPSG = 'CATS2008'
         TYPE = 'z'
-    elif (MODEL == 'TPXO9-atlas'):
+    elif (TIDE_MODEL == 'TPXO9-atlas'):
         model_directory = os.path.join(tide_dir,'TPXO9_atlas')
         grid_file = 'grid_tpxo9_atlas.nc.gz'
         model_files = ['h_q1_tpxo9_atlas_30.nc.gz','h_o1_tpxo9_atlas_30.nc.gz',
@@ -467,13 +478,14 @@ def compute_tides_icebridge_data(tide_dir, arg, MODEL, METHOD=None,
             'h_m4_tpxo9_atlas_30.nc.gz','h_ms4_tpxo9_atlas_30.nc.gz',
             'h_mn4_tpxo9_atlas_30.nc.gz','h_2n2_tpxo9_atlas_30.nc.gz']
         reference = 'http://volkov.oce.orst.edu/tides/tpxo9_atlas.html'
-        attrib['tide']['long_name'] = 'Ocean_Tide'
+        output_variable = 'tide_ocean'
+        variable_long_name = 'Ocean_Tide'
         model_format = 'netcdf'
         TYPE = 'z'
         SCALE = 1.0/1000.0
-    elif (MODEL == 'TPXO9-atlas-v2'):
+    elif (TIDE_MODEL == 'TPXO9-atlas-v2'):
         model_directory = os.path.join(tide_dir,'TPXO9_atlas_v2')
-        grid_file = 'grid_tpxo9_atlas_v2.nc.gz'
+        grid_file = 'grid_tpxo9_atlas_30_v2.nc.gz'
         model_files = ['h_q1_tpxo9_atlas_30_v2.nc.gz','h_o1_tpxo9_atlas_30_v2.nc.gz',
             'h_p1_tpxo9_atlas_30_v2.nc.gz','h_k1_tpxo9_atlas_30_v2.nc.gz',
             'h_n2_tpxo9_atlas_30_v2.nc.gz','h_m2_tpxo9_atlas_30_v2.nc.gz',
@@ -481,79 +493,105 @@ def compute_tides_icebridge_data(tide_dir, arg, MODEL, METHOD=None,
             'h_m4_tpxo9_atlas_30_v2.nc.gz','h_ms4_tpxo9_atlas_30_v2.nc.gz',
             'h_mn4_tpxo9_atlas_30_v2.nc.gz','h_2n2_tpxo9_atlas_30_v2.nc.gz']
         reference = 'https://www.tpxo.net/global/tpxo9-atlas'
-        attrib['tide']['long_name'] = 'Ocean_Tide'
+        output_variable = 'tide_ocean'
+        variable_long_name = 'Ocean_Tide'
         model_format = 'netcdf'
         TYPE = 'z'
         SCALE = 1.0/1000.0
-    elif (MODEL == 'TPXO9.1'):
+    elif (TIDE_MODEL == 'TPXO9-atlas-v3'):
+        model_directory = os.path.join(tide_dir,'TPXO9_atlas_v3')
+        grid_file = 'grid_tpxo9_atlas_30_v3.nc.gz'
+        model_files = ['h_q1_tpxo9_atlas_30_v3.nc.gz','h_o1_tpxo9_atlas_30_v3.nc.gz',
+            'h_p1_tpxo9_atlas_30_v3.nc.gz','h_k1_tpxo9_atlas_30_v3.nc.gz',
+            'h_n2_tpxo9_atlas_30_v3.nc.gz','h_m2_tpxo9_atlas_30_v3.nc.gz',
+            'h_s2_tpxo9_atlas_30_v3.nc.gz','h_k2_tpxo9_atlas_30_v3.nc.gz',
+            'h_m4_tpxo9_atlas_30_v3.nc.gz','h_ms4_tpxo9_atlas_30_v3.nc.gz',
+            'h_mn4_tpxo9_atlas_30_v3.nc.gz','h_2n2_tpxo9_atlas_30_v3.nc.gz',
+            'h_mf_tpxo9_atlas_30_v3.nc.gz','h_mm_tpxo9_atlas_30_v3.nc.gz']
+        reference = 'https://www.tpxo.net/global/tpxo9-atlas'
+        output_variable = 'tide_ocean'
+        variable_long_name = "Ocean Tide"
+        model_format = 'netcdf'
+        TYPE = 'z'
+        SCALE = 1.0/1000.0
+    elif (TIDE_MODEL == 'TPXO9.1'):
         grid_file = os.path.join(tide_dir,'TPXO9.1','DATA','grid_tpxo9')
         model_file = os.path.join(tide_dir,'TPXO9.1','DATA','h_tpxo9.v1')
         reference = 'http://volkov.oce.orst.edu/tides/global.html'
-        attrib['tide']['long_name'] = 'Ocean_Tide'
+        output_variable = 'tide_ocean'
+        variable_long_name = 'Ocean_Tide'
         model_format = 'OTIS'
         EPSG = '4326'
         TYPE = 'z'
-    elif (MODEL == 'TPXO8-atlas'):
+    elif (TIDE_MODEL == 'TPXO8-atlas'):
         grid_file = os.path.join(tide_dir,'tpxo8_atlas','grid_tpxo8atlas_30_v1')
         model_file = os.path.join(tide_dir,'tpxo8_atlas','hf.tpxo8_atlas_30_v1')
         reference = 'http://volkov.oce.orst.edu/tides/tpxo8_atlas.html'
-        attrib['tide']['long_name'] = 'Ocean_Tide'
+        output_variable = 'tide_ocean'
+        variable_long_name = 'Ocean_Tide'
         model_format = 'ATLAS'
         EPSG = '4326'
         TYPE = 'z'
-    elif (MODEL == 'TPXO7.2'):
+    elif (TIDE_MODEL == 'TPXO7.2'):
         grid_file = os.path.join(tide_dir,'TPXO7.2_tmd','grid_tpxo7.2')
         model_file = os.path.join(tide_dir,'TPXO7.2_tmd','h_tpxo7.2')
         reference = 'http://volkov.oce.orst.edu/tides/global.html'
-        attrib['tide']['long_name'] = 'Ocean_Tide'
+        output_variable = 'tide_ocean'
+        variable_long_name = 'Ocean_Tide'
         model_format = 'OTIS'
         EPSG = '4326'
         TYPE = 'z'
-    elif (MODEL == 'TPXO7.2_load'):
+    elif (TIDE_MODEL == 'TPXO7.2_load'):
         grid_file = os.path.join(tide_dir,'TPXO7.2_load','grid_tpxo6.2')
         model_file = os.path.join(tide_dir,'TPXO7.2_load','h_tpxo7.2_load')
         reference = 'http://volkov.oce.orst.edu/tides/global.html'
-        attrib['tide']['long_name'] = 'Load_Tide'
+        output_variable = 'tide_load'
+        variable_long_name = 'Load_Tide'
         model_format = 'OTIS'
         EPSG = '4326'
         TYPE = 'z'
-    elif (MODEL == 'AODTM-5'):
+    elif (TIDE_MODEL == 'AODTM-5'):
         grid_file = os.path.join(tide_dir,'aodtm5_tmd','grid_Arc5km')
         model_file = os.path.join(tide_dir,'aodtm5_tmd','h0_Arc5km.oce')
         reference = ('https://www.esr.org/research/polar-tide-models/'
             'list-of-polar-tide-models/aodtm-5/')
-        attrib['tide']['long_name'] = 'Ocean_Tide'
+        output_variable = 'tide_ocean'
+        variable_long_name = 'Ocean_Tide'
         model_format = 'OTIS'
         EPSG = 'PSNorth'
         TYPE = 'z'
-    elif (MODEL == 'AOTIM-5'):
+    elif (TIDE_MODEL == 'AOTIM-5'):
         grid_file = os.path.join(tide_dir,'aotim5_tmd','grid_Arc5km')
         model_file = os.path.join(tide_dir,'aotim5_tmd','h_Arc5km.oce')
         reference = ('https://www.esr.org/research/polar-tide-models/'
             'list-of-polar-tide-models/aotim-5/')
-        attrib['tide']['long_name'] = 'Ocean_Tide'
+        output_variable = 'tide_ocean'
+        variable_long_name = 'Ocean_Tide'
         model_format = 'OTIS'
         EPSG = 'PSNorth'
         TYPE = 'z'
-    elif (MODEL == 'AOTIM-5-2018'):
+    elif (TIDE_MODEL == 'AOTIM-5-2018'):
         grid_file = os.path.join(tide_dir,'Arc5km2018','grid_Arc5km2018')
         model_file = os.path.join(tide_dir,'Arc5km2018','h_Arc5km2018')
         reference = ('https://www.esr.org/research/polar-tide-models/'
             'list-of-polar-tide-models/aotim-5/')
+        output_variable = 'tide_ocean'
+        variable_long_name = 'Ocean_Tide'
         model_format = 'OTIS'
         EPSG = 'PSNorth'
         TYPE = 'z'
-    elif (MODEL == 'GOT4.7'):
+    elif (TIDE_MODEL == 'GOT4.7'):
         model_directory = os.path.join(tide_dir,'GOT4.7','grids_oceantide')
         model_files = ['q1.d.gz','o1.d.gz','p1.d.gz','k1.d.gz','n2.d.gz',
             'm2.d.gz','s2.d.gz','k2.d.gz','s1.d.gz','m4.d.gz']
         c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
         reference = ('https://denali.gsfc.nasa.gov/personal_pages/ray/'
             'MiscPubs/19990089548_1999150788.pdf')
-        attrib['tide']['long_name'] = 'Ocean_Tide'
+        output_variable = 'tide_ocean'
+        variable_long_name = 'Ocean_Tide'
         model_format = 'GOT'
         SCALE = 1.0/100.0
-    elif (MODEL == 'GOT4.7_load'):
+    elif (TIDE_MODEL == 'GOT4.7_load'):
         model_directory = os.path.join(tide_dir,'GOT4.7','grids_loadtide')
         model_files = ['q1load.d.gz','o1load.d.gz','p1load.d.gz','k1load.d.gz',
             'n2load.d.gz','m2load.d.gz','s2load.d.gz','k2load.d.gz',
@@ -561,20 +599,22 @@ def compute_tides_icebridge_data(tide_dir, arg, MODEL, METHOD=None,
         c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
         reference = ('https://denali.gsfc.nasa.gov/personal_pages/ray/'
             'MiscPubs/19990089548_1999150788.pdf')
-        attrib['tide']['long_name'] = 'Load_Tide'
+        output_variable = 'tide_load'
+        variable_long_name = 'Load_Tide'
         model_format = 'GOT'
         SCALE = 1.0/1000.0
-    elif (MODEL == 'GOT4.8'):
+    elif (TIDE_MODEL == 'GOT4.8'):
         model_directory = os.path.join(tide_dir,'got4.8','grids_oceantide')
         model_files = ['q1.d.gz','o1.d.gz','p1.d.gz','k1.d.gz','n2.d.gz',
             'm2.d.gz','s2.d.gz','k2.d.gz','s1.d.gz','m4.d.gz']
         c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
         reference = ('https://denali.gsfc.nasa.gov/personal_pages/ray/'
             'MiscPubs/19990089548_1999150788.pdf')
-        attrib['tide']['long_name'] = 'Ocean_Tide'
+        output_variable = 'tide_ocean'
+        variable_long_name = 'Ocean_Tide'
         model_format = 'GOT'
         SCALE = 1.0/100.0
-    elif (MODEL == 'GOT4.8_load'):
+    elif (TIDE_MODEL == 'GOT4.8_load'):
         model_directory = os.path.join(tide_dir,'got4.8','grids_loadtide')
         model_files = ['q1load.d.gz','o1load.d.gz','p1load.d.gz','k1load.d.gz',
             'n2load.d.gz','m2load.d.gz','s2load.d.gz','k2load.d.gz',
@@ -582,20 +622,22 @@ def compute_tides_icebridge_data(tide_dir, arg, MODEL, METHOD=None,
         c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
         reference = ('https://denali.gsfc.nasa.gov/personal_pages/ray/'
             'MiscPubs/19990089548_1999150788.pdf')
-        attrib['tide']['long_name'] = 'Load_Tide'
+        output_variable = 'tide_load'
+        variable_long_name = 'Load_Tide'
         model_format = 'GOT'
         SCALE = 1.0/1000.0
-    elif (MODEL == 'GOT4.10'):
+    elif (TIDE_MODEL == 'GOT4.10'):
         model_directory = os.path.join(tide_dir,'GOT4.10c','grids_oceantide')
         model_files = ['q1.d.gz','o1.d.gz','p1.d.gz','k1.d.gz','n2.d.gz',
             'm2.d.gz','s2.d.gz','k2.d.gz','s1.d.gz','m4.d.gz']
         c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
         reference = ('https://denali.gsfc.nasa.gov/personal_pages/ray/'
             'MiscPubs/19990089548_1999150788.pdf')
-        attrib['tide']['long_name'] = 'Ocean_Tide'
+        output_variable = 'tide_ocean'
+        variable_long_name = 'Ocean_Tide'
         model_format = 'GOT'
         SCALE = 1.0/100.0
-    elif (MODEL == 'GOT4.10_load'):
+    elif (TIDE_MODEL == 'GOT4.10_load'):
         model_directory = os.path.join(tide_dir,'GOT4.10c','grids_loadtide')
         model_files = ['q1load.d.gz','o1load.d.gz','p1load.d.gz','k1load.d.gz',
             'n2load.d.gz','m2load.d.gz','s2load.d.gz','k2load.d.gz',
@@ -603,10 +645,11 @@ def compute_tides_icebridge_data(tide_dir, arg, MODEL, METHOD=None,
         c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
         reference = ('https://denali.gsfc.nasa.gov/personal_pages/ray/'
             'MiscPubs/19990089548_1999150788.pdf')
-        attrib['tide']['long_name'] = 'Load_Tide'
+        output_variable = 'tide_load'
+        variable_long_name = 'Load_Tide'
         model_format = 'GOT'
         SCALE = 1.0/1000.0
-    elif (MODEL == 'FES2014'):
+    elif (TIDE_MODEL == 'FES2014'):
         model_directory = os.path.join(tide_dir,'fes2014','ocean_tide')
         model_files = ['2n2.nc.gz','eps2.nc.gz','j1.nc.gz','k1.nc.gz',
             'k2.nc.gz','l2.nc.gz','la2.nc.gz','m2.nc.gz','m3.nc.gz','m4.nc.gz',
@@ -620,11 +663,12 @@ def compute_tides_icebridge_data(tide_dir, arg, MODEL, METHOD=None,
             'n4','nu2','o1','p1','q1','r2','s1','s2','s4','sa','ssa','t2']
         reference = ('https://www.aviso.altimetry.fr/data/products/'
             'auxiliary-products/global-tide-fes.html')
-        attrib['tide']['long_name'] = 'Ocean_Tide'
+        output_variable = 'tide_ocean'
+        variable_long_name = 'Ocean_Tide'
         model_format = 'FES'
         TYPE = 'z'
         SCALE = 1.0/100.0
-    elif (MODEL == 'FES2014_load'):
+    elif (TIDE_MODEL == 'FES2014_load'):
         model_directory = os.path.join(tide_dir,'fes2014','load_tide')
         model_files = ['2n2.nc.gz','eps2.nc.gz','j1.nc.gz','k1.nc.gz',
             'k2.nc.gz','l2.nc.gz','la2.nc.gz','m2.nc.gz','m3.nc.gz','m4.nc.gz',
@@ -638,31 +682,41 @@ def compute_tides_icebridge_data(tide_dir, arg, MODEL, METHOD=None,
             'n4','nu2','o1','p1','q1','r2','s1','s2','s4','sa','ssa','t2']
         reference = ('https://www.aviso.altimetry.fr/data/products/'
             'auxiliary-products/global-tide-fes.html')
-        attrib['tide']['long_name'] = 'Load_Tide'
+        output_variable = 'tide_load'
+        variable_long_name = 'Load_Tide'
         model_format = 'FES'
         TYPE = 'z'
         SCALE = 1.0/100.0
 
+    #-- HDF5 file attributes
+    attrib = {}
     #-- latitude
+    attrib['lat'] = {}
     attrib['lat']['long_name'] = 'Latitude_of_measurement'
     attrib['lat']['description'] = ('Corresponding_to_the_measurement_'
         'position_at_the_acquisition_time')
     attrib['lat']['units'] = 'Degrees_North'
     #-- longitude
+    attrib['lon'] = {}
     attrib['lon']['long_name'] = 'Longitude_of_measurement'
     attrib['lon']['description'] = ('Corresponding_to_the_measurement_'
         'position_at_the_acquisition_time')
     attrib['lon']['units'] = 'Degrees_East'
     #-- tides
-    attrib['tide']['description'] = ('Tidal_elevation_from_harmonic_constants_'
-        'at_the_measurement_position_at_the_acquisition_time')
-    attrib['tide']['reference'] = reference
-    attrib['tide']['model'] = MODEL
-    attrib['tide']['units'] = 'meters'
-    #-- days
-    attrib['day']['long_name'] = 'Time'
-    attrib['day']['description'] = 'Days relative to 1992-01-01 (MJD:48622)'
-    attrib['day']['units'] = 'Days'
+    attrib[output_variable] = {}
+    attrib[output_variable]['description'] = ('Tidal_elevation_from_harmonic_'
+        'constants_at_the_measurement_position_at_the_acquisition_time')
+    attrib[output_variable]['reference'] = reference
+    attrib[output_variable]['model'] = TIDE_MODEL
+    attrib[output_variable]['units'] = 'meters'
+    attrib[output_variable]['long_name'] = variable_long_name
+    #-- time
+    attrib['time'] = {}
+    attrib['time']['long_name'] = 'Time'
+    attrib['time']['description'] = ('Time_corresponding_to_the_measurement_'
+        'position')
+    attrib['time']['units'] = 'Days since 1992-01-01T00:00:00'
+    attrib['time']['calendar'] = 'standard'
 
     #-- extract information from first input file
     #-- acquisition year, month and day
@@ -716,13 +770,15 @@ def compute_tides_icebridge_data(tide_dir, arg, MODEL, METHOD=None,
         amp,ph = extract_GOT_constants(dinput['lon'], dinput['lat'],
             model_directory, model_files, METHOD=METHOD, SCALE=SCALE)
         #-- interpolate delta times from calendar dates to tide time
-        delta_file = pyTMD.utilities.get_data_path(['data','merged_deltat.data'])
+        delta_file = get_data_path(['data','merged_deltat.data'])
         deltat = calc_delta_time(delta_file, t)
     elif (model_format == 'FES'):
         amp,ph = extract_FES_constants(dinput['lon'], dinput['lat'],
-            model_directory, model_files, TYPE=TYPE, VERSION=MODEL,
+            model_directory, model_files, TYPE=TYPE, VERSION=TIDE_MODEL,
             METHOD=METHOD, SCALE=SCALE)
-        deltat = np.zeros_like(t)
+        #-- interpolate delta times from calendar dates to tide time
+        delta_file = get_data_path(['data','merged_deltat.data'])
+        deltat = calc_delta_time(delta_file, t)
 
     #-- calculate complex phase in radians for Euler's
     cph = -1j*ph*np.pi/180.0
@@ -732,7 +788,7 @@ def compute_tides_icebridge_data(tide_dir, arg, MODEL, METHOD=None,
     #-- output tidal HDF5 file
     #-- form: rg_NASA_model_TIDES_WGS84_fl1yyyymmddjjjjj.H5
     #-- where rg is the hemisphere flag (GR or AN) for the region
-    #-- model is the tidal MODEL flag (e.g. CATS0201)
+    #-- model is the tidal TIDE_MODEL flag (e.g. CATS0201)
     #-- fl1 and fl2 are the data flags (ATM, LVIS, GLAS)
     #-- yymmddjjjjj is the year, month, day and second of the input file
     #-- output region flags: GR for Greenland and AN for Antarctica
@@ -740,7 +796,7 @@ def compute_tides_icebridge_data(tide_dir, arg, MODEL, METHOD=None,
     #-- use starting second to distinguish between files for the day
     JJ1 = np.min(dinput['time']) % 86400
     #-- output file format
-    args = (hem_flag[HEM],MODEL,OIB,YY1,MM1,DD1,JJ1)
+    args = (hem_flag[HEM],TIDE_MODEL,OIB,YY1,MM1,DD1,JJ1)
     FILENAME = '{0}_NASA_{1}_TIDES_WGS84_{2}{3}{4}{5}{6:05.0f}.H5'.format(*args)
     #-- print file information
     print('\t{0}'.format(FILENAME)) if VERBOSE else None
@@ -772,22 +828,22 @@ def compute_tides_icebridge_data(tide_dir, arg, MODEL, METHOD=None,
         h5.dims[0].label = 'RECORD_SIZE'
 
     #-- output tides to HDF5 dataset
-    h5 = fid.create_dataset('tide', (file_lines,), data=tide,
+    h5 = fid.create_dataset(output_variable, (file_lines,), data=tide,
         dtype=tide.dtype, fillvalue=fill_value, compression='gzip')
     #-- add HDF5 variable attributes
     tide_count = np.count_nonzero(tide != fill_value)
     h5.attrs['tide_count'] = tide_count
     h5.attrs['_FillValue'] = fill_value
-    for att_name,att_val in attrib['tide'].items():
+    for att_name,att_val in attrib[output_variable].items():
         h5.attrs[att_name] = att_val
     #-- attach dimensions
     h5.dims[0].label = 'RECORD_SIZE'
 
     #-- output days to HDF5 dataset
-    h5 = fid.create_dataset('day', (file_lines,), data=t,
+    h5 = fid.create_dataset('time', (file_lines,), data=t,
         dtype=t.dtype, compression='gzip')
     #-- add HDF5 variable attributes
-    for att_name,att_val in attrib['day'].items():
+    for att_name,att_val in attrib['time'].items():
         h5.attrs[att_name] = att_val
     #-- attach dimensions
     h5.dims[0].label = 'RECORD_SIZE'
@@ -802,7 +858,7 @@ def compute_tides_icebridge_data(tide_dir, arg, MODEL, METHOD=None,
     fid.attrs['date_created'] = time.strftime('%Y-%m-%d',time.localtime())
     #-- add attributes for input files
     fid.attrs['elevation_file'] = os.path.basename(input_file)
-    fid.attrs['tide_model'] = MODEL
+    fid.attrs['tide_model'] = TIDE_MODEL
     #-- add geospatial and temporal attributes
     fid.attrs['geospatial_lat_min'] = dinput['lat'].min()
     fid.attrs['geospatial_lat_max'] = dinput['lat'].max()
@@ -834,61 +890,54 @@ def compute_tides_icebridge_data(tide_dir, arg, MODEL, METHOD=None,
     #-- change the permissions level to MODE
     os.chmod(os.path.join(DIRECTORY,FILENAME), MODE)
 
-#-- PURPOSE: help module to describe the optional input parameters
-def usage():
-    print('\nHelp: {}'.format(os.path.basename(sys.argv[0])))
-    print(' -D X, --directory=X\tWorking data directory')
-    print(' -T X, --tide=X\t\tTide model to use in correction')
-    print(' -I X, --interpolate\tInterpolation method (default spline)')
-    print(' -M X, --mode=X\t\tPermission mode of directories and files created')
-    print(' -V, --verbose\t\tOutput information about each created file\n')
-
 #-- Main program that calls compute_tides_icebridge_data()
 def main():
     #-- Read the system arguments listed after the program
-    long_options = ['help','directory=','tide=','interpolate=','verbose','mode=']
-    optlist,arglist = getopt.getopt(sys.argv[1:], 'hD:T:I:VM:', long_options)
-
+    parser = argparse.ArgumentParser(
+        description="""Calculates tidal elevations for correcting Operation
+            IceBridge elevation data
+            """
+    )
+    #-- command line parameters
+    parser.add_argument('infile',
+        type=lambda p: os.path.abspath(os.path.expanduser(p)), nargs='+',
+        help='Input Operation IceBridge file')
     #-- directory with tide data
-    tide_dir = os.getcwd()
+    parser.add_argument('--directory','-D',
+        type=lambda p: os.path.abspath(os.path.expanduser(p)),
+        default=os.getcwd(),
+        help='Working data directory')
     #-- tide model to use
-    MODEL = 'CATS2008'
+    model_choices = ('CATS0201','CATS2008','CATS2008_load',
+        'TPXO9-atlas','TPXO9-atlas-v2','TPXO9-atlas-v3',
+        'TPXO9.1','TPXO8-atlas','TPXO7.2','TPXO7.2_load',
+        'AODTM-5','AOTIM-5','AOTIM-5-2018',
+        'GOT4.7','GOT4.7_load','GOT4.8','GOT4.8_load','GOT4.10','GOT4.10_load',
+        'FES2014','FES2014_load')
+    parser.add_argument('--tide','-T',
+        metavar='TIDE', type=str, default='CATS2008',
+        choices=model_choices,
+        help='Tide model to use in correction')
     #-- interpolation method
-    METHOD = 'spline'
+    parser.add_argument('--interpolate','-I',
+        metavar='METHOD', type=str, default='spline',
+        choices=('spline','linear','nearest','bilinear'),
+        help='Spatial interpolation method')
     #-- verbosity settings
-    VERBOSE = False
+    #-- verbose will output information about each output file
+    parser.add_argument('--verbose','-V',
+        default=False, action='store_true',
+        help='Output information about each created file')
     #-- permissions mode of the local files (number in octal)
-    MODE = 0o775
-    for opt, arg in optlist:
-        if opt in ('-h','--help'):
-            usage()
-            sys.exit()
-        elif opt in ("-D","--directory"):
-            tide_dir = os.path.expanduser(arg)
-        elif opt in ("-T","--tide"):
-            MODEL = arg
-        elif opt in ("-I","--interpolate"):
-            METHOD = arg.lower()
-        elif opt in ("-V","--verbose"):
-            VERBOSE = True
-        elif opt in ("-M","--mode"):
-            MODE = int(arg, 8)
+    parser.add_argument('--mode','-M',
+        type=lambda x: int(x,base=8), default=0o775,
+        help='Permission mode of directories and files created')
+    args = parser.parse_args()
 
-    #-- enter input file from NSIDC as system argument
-    if not arglist:
-        raise Exception('No System Arguments Listed')
-
-    #-- verify model before running program
-    model_list = ['CATS0201','CATS2008','CATS2008_load','TPXO9-atlas','TPXO9.1',
-        'TPXO8-atlas','TPXO7.2','TPXO7.2_load','AODTM-5','AOTIM-5',
-        'AOTIM-5-2018','GOT4.7','GOT4.7_load','GOT4.8','GOT4.8_load',
-        'GOT4.10','GOT4.10_load','FES2014','FES2014_load']
-    assert MODEL in model_list, 'Unlisted tide model'
-
-    #-- run for each input file
-    for arg in arglist:
-        compute_tides_icebridge_data(tide_dir, os.path.expanduser(arg), MODEL,
-            METHOD=METHOD, VERBOSE=VERBOSE,MODE=MODE)
+    #-- run for each input Operation IceBridge file
+    for arg in args.infile:
+        compute_tides_icebridge_data(args.directory, arg, TIDE_MODEL=args.tide,
+            METHOD=args.interpolate, VERBOSE=args.verbose, MODE=args.mode)
 
 #-- run main program
 if __name__ == '__main__':
