@@ -2,9 +2,11 @@
 u"""
 test_leap_seconds.py (08/2020)
 """
-import warnings
+import os
 import pytest
+import warnings
 import pyTMD.time
+import pyTMD.utilities
 
 #-- PURPOSE: Define GPS leap seconds
 def get_leaps():
@@ -21,3 +23,9 @@ def test_leap_seconds():
     valid_gps_leaps = get_leaps()
     test_gps_leaps = pyTMD.time.get_leap_seconds()
     assert all((v==t) for v,t in zip(valid_gps_leaps,test_gps_leaps))
+
+#-- PURPOSE: update leap second file
+def test_update_leap_seconds():
+    pyTMD.time.update_leap_seconds(verbose=False, mode=0o775)
+    FILE = pyTMD.utilities.get_data_path(['data','leap-seconds.list'])
+    assert os.access(FILE,os.F_OK)
